@@ -175,9 +175,15 @@ EXTERN char *dosemu_midi_path INIT("~/" LOCALDIR_BASE_NAME "/run/" DOSEMU_MIDI);
  */
 #undef RUN_AS_ROOT
 
+/* Define this to use threading code */
+#undef USE_THREADS
+
 /* Define this to use the new interrupt handling code */
 #undef USE_NEW_INT
 
+/* Define this to use the X86 CPU emulator */
+#undef X86_EMULATOR
+
 @BOTTOM@
 
 #endif /* CONFIG_H */
