@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -44,7 +44,12 @@ extern int type_in_pre_strokes();
 extern int errno;
 #endif
 
+#ifdef X86_EMULATOR
+extern int e_sig_pending;
+#define E_SIGNAL	if (config.cpuemu>1) e_sig_pending=1
+#else
 #define E_SIGNAL
+#endif
 
 /* Variables for keeping track of signals */
 #define MAX_SIG_QUEUE_SIZE 50
@@ -60,7 +65,7 @@ static struct SIGNAL_queue signal_queue[MAX_SIG_QUEUE_SIZE];
 static sigset_t oldset;
 
 #ifdef __linux__
-#if GLIBC_VERSION_CODE >= 2000
+#if __GLIBC__ > 1		/* glibc-2 version */
 /*
  * Thomas Winder <thomas.winder@sea.ericsson.se> wrote:
  * glibc-2 uses a different struct sigaction type than the one used in
@@ -310,7 +315,9 @@ void SIGALRM_call(void)
 {
   static int first = 0;
   static hitimer_t cnt200 = 0;
+#if !defined(USE_THREADS)
   static hitimer_t cnt1000 = 0;
+#endif
   static volatile int running = 0;
 #if VIDEO_CHECK_DIRTY
   static int update_pending = 0;
@@ -319,7 +326,9 @@ void SIGALRM_call(void)
   
   if (first==0) {
     cnt200 =
+#if !defined(USE_THREADS)
     cnt1000 =
+#endif
     pic_sys_time;	/* initialize */
     first = 1;
   }
@@ -477,12 +486,14 @@ void SIGALRM_call(void)
 
 /* We update the RTC from here if it has not been defined as a thread */
 
+#if !defined(USE_THREADS)
   /* this is for EXACT per-second activities (can produce bursts) */
   if ((pic_sys_time-cnt1000) >= PIT_TICK_RATE) {
     cnt1000 += PIT_TICK_RATE;
 /*    g_printf("**** ALRM: 1sec\n"); */
     rtc_update();
   }
+#endif
 
 }
 
@@ -568,6 +579,15 @@ sigalrm(int sig, struct sigcontext_struct context)
 }
 #endif
 
+#ifdef X86_EMULATOR
+void
+e_sigalrm(void)
+{
+/*  if (in_dpmi && !in_vm86)
+    dpmi_sigio(&context); */
+  SIGNAL_save(SIGALRM_call);
+}
+#endif
 
 #ifdef __NetBSD__
 void
