@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -1326,7 +1326,11 @@ static void mhp_regs(int argc, char * argv[])
 
  if (!traceloop)
   mhp_printf( "system state: %s%s%s%s\n",
+#ifdef X86_EMULATOR
+       config.cpuemu > 1 ? "emulated," : "",
+#else
        "",
+#endif
        mhpdbgc.stopped ? "stopped" : "running",
        IN_DPMI ? " in DPMI" : (in_dpmi?" in real mode while in DPMI":""),
        IN_DPMI ?(dpmi_mhp_getcsdefault()?"-32bit":"-16bit") : "");
