@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -63,9 +63,6 @@ static unsigned long  precard_eip, precard_cs;
 
 static struct timeval scr_tv;        /* For translating UNIX <-> DOS times */
 
-/* set if some directories are mounted during startup */
-int redir_state = 0;
-
 #ifdef USE_MRP_JOYSTICK
 #include <linux/joystick.h>
 #include <fcntl.h>
@@ -192,6 +189,10 @@ static void process_master_boot_record(void)
 /* returns 1 if dos_helper() handles it, 0 otherwise */
 static int dos_helper(void)
 {
+#ifdef X86_EMULATOR
+  extern void enter_cpu_emu(void);
+  extern void leave_cpu_emu(void);
+#endif
 
   switch (LO(ax)) {
   case DOS_HELPER_DOSEMU_CHECK:			/* Linux dosemu installation test */
@@ -215,7 +216,7 @@ static int dos_helper(void)
     show_ints(HI(bx), LO(bx));
     break;
 
-  case DOS_HELPER_PRINT_STRING:	/* PRINT STRING ES:DX */
+  case DOS_HELPER_PRINT_STRING:	/* SHOW INTS, BH-BL */
     g_printf("DOS likes us to print a string\n");
     ds_printf("DOS to EMU: \"%s\"\n",SEG_ADR((char *), es, dx));
     break;
@@ -369,18 +370,6 @@ static int dos_helper(void)
     mfs_inte6();
     return 1;
 
-  case DOS_HELPER_DOSC:
-    if (HI(ax) == 0xdc) {
-      /* install check and notify */
-      if (!dosc_interface()) return 0;
-      running_DosC = LWORD(ebx);
-      return 1;
-    }
-    if (running_DosC) {
-      return dosc_interface();
-    }
-    return 0;
-
   case DOS_HELPER_EMS_HELPER:
     ems_helper();
     return 1;
@@ -434,18 +423,6 @@ static int dos_helper(void)
       break;
     }
 
-  case DOS_HELPER_ASPI_HELPER: {
-#ifdef ASPI_SUPPORT
-      extern void aspi_helper(int);
-      A_printf("ASPI: in 0x41 handler! ax=0x%04x, bx=0x%04x, dx=0x%04x, "
-           "cx=0x%04x\n", LWORD(eax), LWORD(ebx), LWORD(edx), LWORD(ecx));
-      aspi_helper(HI(ax));
-#else
-      LWORD(eax) = 0;
-#endif
-      break;
-  }
-
   case DOS_HELPER_RUN_UNIX:
     g_printf("Running Unix Command\n");
     run_unix_command(SEG_ADR((char *), es, dx));
@@ -464,9 +441,16 @@ static int dos_helper(void)
     break;   
 
   case DOS_HELPER_0x53:
+    if (config.secure) {
+      CARRY;
+      LWORD(eax) = 0;
+      break;
+    }
     {
-        extern int run_system_command(char *);
-        LWORD(eax) = run_system_command(SEG_ADR((char *), es, dx));
+	PRIV_SAVE_AREA
+	enter_priv_off();
+	LWORD(eax) = system(SEG_ADR((char *), es, dx));
+	leave_priv_setting();
 	break;
     }
 
@@ -490,6 +474,14 @@ static int dos_helper(void)
   case DOS_HELPER_CHDIR:
         LWORD(eax) = chdir(SEG_ADR((char *), es, dx));
         break;
+#ifdef X86_EMULATOR
+  case DOS_HELPER_CPUEMUON:
+  	if (config.cpuemu && !in_dpmi) enter_cpu_emu();
+        break;
+  case DOS_HELPER_CPUEMUOFF:
+  	if (config.cpuemu && !in_dpmi) leave_cpu_emu();
+        break;
+#endif
     case DOS_HELPER_XCONFIG:
 #if X_GRAPHICS
 	if (config.X) {
@@ -653,7 +645,7 @@ static void int15(u_char i)
   case 0xc0:
     LWORD(es) = ROM_CONFIG_SEG;
     LWORD(ebx) = ROM_CONFIG_OFF;
-    HI(ax) = 0;
+    LO(ax) = 0;
     return;
   case 0xc1:
     CARRY;
@@ -811,35 +803,6 @@ Notes:	there are approximately 18.2 clock ticks per second, 1800B0h per 24 hrs
 ->	  midnight flag and will fail to advance the date
 */
   case 0:			/* read time counter */
-
-#define BIOSTIMER_ONLY_VIEW
-#ifdef BIOSTIMER_ONLY_VIEW
-
-    /* We rely on the INT8 routine doing the right thing,
-     * DOS apps too rely on the relationship between INT1A and 0x46c timer.
-     * We already do all appropriate things to trigger the simulated INT8
-     * correctly (well, sometimes faking it), so the 0x46c timer incremented
-     * by the (realmode) INT8 handler should be always in sync.
-     * Therefore, we keep INT1A,AH0 simple instead of trying to be too clever;-)
-     */
-    {
-      static int first = 1;
-      if (first) {
-        /* take over the correct value _once_ only */
-        *((unsigned long *)(BIOS_TICK_ADDR)) =
-               (unsigned long)(pic_sys_time >> 16)
-             + (sys_base_ticks + usr_delta_ticks);
-        first = 0;
-      }
-    }
-    last_ticks = *((unsigned long *)(BIOS_TICK_ADDR));
-    LO(ax) = *((u_char *)(TICK_OVERFLOW_ADDR));
-    LWORD(ecx) = (last_ticks >> 16) & 0xffff;
-    LWORD(edx) = last_ticks & 0xffff;
-    *((u_char *)(TICK_OVERFLOW_ADDR)) = 0; /* clear the midnight flag */
-
-#else /* not BIOSTIMER_ONLY_VIEW */
-+
     /* it works because pic_sys_time has a zero reference. It doesn't
      * account for timer speedups, but any decent DOS program should
      * call back the original int8 at the 18.2 Hz rate... */
@@ -855,7 +818,6 @@ Notes:	there are approximately 18.2 clock ticks per second, 1800B0h per 24 hrs
     LWORD(edx) = last_ticks & 0xffff;
     g_printf("TIMER: read timer = %lu\n", last_ticks);
     set_ticks(last_ticks);	/* set_ticks is in rtc.c */
-#endif /* not BIOSTIMER_ONLY_VIEW */
     break;
 
 /*
@@ -1071,7 +1033,7 @@ static int ms_dos(int nr)
     char *ptr = SEG_ADR((char *), ds, dx);
 
     /* ignore explicitly selected drive by incrementing ptr by 1 */
-    if (config.emubat && !strncasecmp(ptr + 1, ":\\AUTOEXEC.BAT", 14)) {
+    if (config.emubat && !strncmp(ptr + 1, ":\\AUTOEXEC.BAT", 14)) {
       ext_fix(config.emubat);
       sprintf(ptr + 1, ":\\AUTOEXEC.%-3s", config.emubat);
       d_printf("DISK: Substituted %s for AUTOEXEC.BAT\n", ptr + 1);
@@ -1138,11 +1100,9 @@ static int ms_dos(int nr)
     return 0;
 
   default:
-#ifndef USE_NEW_INT
     if (!in_dpmi)
       g_printf("INT21 (0x%02x):  we shouldn't be here! ax=0x%04x, bx=0x%04x\n",
 	     nr, LWORD(eax), LWORD(ebx));
-#endif
     return 0;
   }
   return 1;
@@ -1428,146 +1388,11 @@ static void int19(u_char i) {
   boot();
 }
 
-/*
- * Turn all simulated FAT devices into network drives.
- */
-static void redirect_devices()
-{
-  extern int RedirectDisk(int, char *, int);
-
-  static char s[256] = "\\\\LINUX\\FS", *t = s + 10;
-  int i, j;
-
-  for (i = 0; i < MAX_HDISKS; i++) {
-    if(hdisktab[i].type == DIR_TYPE && hdisktab[i].fatfs) {
-      strncpy(t, hdisktab[i].dev_name, 245);
-      s[255] = 0;
-      j = RedirectDisk(i + 2, s, hdisktab[i].rdonly);
-
-      ds_printf("INT21: redirecting %c: %s (err = %d)\n", i + 'C', j ? "failed" : "ok", j);
-    }
-  }
-}
-
-/*
- * Activate the redirector just before the first int 21h file open call.
- *
- * To use this feature, set redir_state = 1 and make sure int 21h is
- * revectored.
- */
-static int redir_it()
-{
-  /*
-   * Declaring the following struct volatile works around an EGCS bug
-   * (at least up to egcs-2.91.66). Otherwise the line below marked
-   * with '###' will modify the *old* (saved) struct too.
-   * -- sw
-   */
-  volatile static struct vm86_regs save_regs;
-  static unsigned x0, x1, x2, x3, x4;
-  unsigned u;
-
-  /*
-   * To start up the redirector we need (1) the list of list, (2) the DOS version and
-   * (3) the swappable data area. To get these, we reuse the original file open call.
-   */
-  switch(redir_state) {
-    case 1:
-      if(HI(ax) == 0x3d) {
-        /*
-         * FreeDOS will get confused by the following calling sequence (e.i. it
-         * is not reentrant 'enough'. So we will abort here - it cannot use a
-         * redirector anyway.
-         * -- sw
-         */
-        if (running_DosC) {
-          ds_printf("INT21: FreeDOS detected - no check for redirector\n");
-          return redir_state = 0;
-        }
-        save_regs = REGS;
-        redir_state = 2;
-        LWORD(eip) -= 2;
-        LWORD(eax) = 0x5200;		/* ### , see above EGCS comment! */
-        default_interrupt(0x21);
-        ds_printf("INT21 +1 (%d) at %04x:%04x: AX=%04x, BX=%04x, CX=%04x, DX=%04x, DS=%04x, ES=%04x\n",
-          redir_state, LWORD(cs), LWORD(eip), LWORD(eax), LWORD(ebx), LWORD(ecx), LWORD(edx), LWORD(ds), LWORD(es));
-        return 1;
-      }
-      break;
-
-    case 2:
-      x0 = LWORD(ebx); x1 = REG(es);
-      redir_state = 3;
-      LWORD(eip) -= 2;
-      LWORD(eax) = 0x3000;
-      default_interrupt(0x21);
-      ds_printf("INT21 +2 (%d) at %04x:%04x: AX=%04x, BX=%04x, CX=%04x, DX=%04x, DS=%04x, ES=%04x\n",
-        redir_state, LWORD(cs), LWORD(eip), LWORD(eax), LWORD(ebx), LWORD(ecx), LWORD(edx), LWORD(ds), LWORD(es));
-      return 2;
-      break;
-
-    case 3:
-      x4 = LWORD(eax);
-      redir_state = 4;
-      LWORD(eip) -= 2;
-      LWORD(eax) = 0x5d06;
-      default_interrupt(0x21);
-      ds_printf("INT21 +3 (%d) at %04x:%04x: AX=%04x, BX=%04x, CX=%04x, DX=%04x, DS=%04x, ES=%04x\n",
-        redir_state, LWORD(cs), LWORD(eip), LWORD(eax), LWORD(ebx), LWORD(ecx), LWORD(edx), LWORD(ds), LWORD(es));
-      return 3;
-      break;
-
-    case 4:
-      x2 = LWORD(esi); x3 = REG(ds);
-      redir_state = 0;
-      u = x0 + (x1 << 4);
-      ds_printf("INT21: lol = 0x%x\n", u);
-      ds_printf("INT21: sda = 0x%x\n", x2 + (x3 << 4));
-      ds_printf("INT21: ver = 0x%02x\n", x4);
-
-      if(*(unsigned *) (u + 0x16)) {		/* Do we have a CDS entry? */
-        /* Init the redirector. */
-        LWORD(ecx) = x4;
-        LWORD(edx) = x0; REG(es) = x1;
-        LWORD(esi) = x2; REG(ds) = x3;
-        LWORD(ebx) = 0x500;
-        LWORD(eax) = 0x20;
-        mfs_inte6();
-
-        redirect_devices();
-      }
-      else {
-        ds_printf("INT21: this DOS has no CDS entry - redirector not used\n");
-      }
-
-      REGS = save_regs;
-      set_int21_revectored(-1);
-      break;
-  }
-
-  return 0;
-}
-
-
 /* MS-DOS */
 static void int21(u_char i) {
-  ds_printf("INT21 (%d) at %04x:%04x: AX=%04x, BX=%04x, CX=%04x, DX=%04x, DS=%04x, ES=%04x\n",
-       redir_state, LWORD(cs), LWORD(eip),
+  ds_printf("INT21 at %04x:%04x: AX=%04x, BX=%04x, CX=%04x, DX=%04x, DS=%04x, ES=%04x\n",
+       LWORD(cs), LWORD(eip),
        LWORD(eax), LWORD(ebx), LWORD(ecx), LWORD(edx), LWORD(ds), LWORD(es));
-
-  if(redir_state && redir_it()) return;
-
-#if 1
-  if(HI(ax) == 0x3d) {
-    char *p = (char *) (((REG(ds)) << 4) + LWORD(edx));
-    int i;
-
-    ds_printf("INT21: open file \"");
-    for(i = 0; i < 64 && p[i]; i++) ds_printf("%c", p[i]);
-    ds_printf("\"\n");
-  }
-#endif
-
   if (!ms_dos(HI(ax)))
     default_interrupt(i);
 }
@@ -1660,11 +1485,6 @@ static void int29(u_char i) {
 
 static void int2f(u_char i)
 {
-#if 1
-  ds_printf("INT2F at %04x:%04x: AX=%04x, BX=%04x, CX=%04x, DX=%04x, DS=%04x, ES=%04x\n",
-       LWORD(cs), LWORD(eip),
-       LWORD(eax), LWORD(ebx), LWORD(ecx), LWORD(edx), LWORD(ds), LWORD(es));
-#endif
   switch (LWORD(eax)) {
   case INT2F_IDLE_MAGIC:   /* magic "give up time slice" value */
     if (config.hogthreshold)
@@ -2251,26 +2071,18 @@ void setup_interrupts(void) {
 #endif /* not USE_NEW_INT */
 }
 
+static struct revectored_struct saved_int21_revectored = {{0,0,0,0,0,0,0,0}};
 
-void set_int21_revectored(int a)
+void set_int21_revectored(int all)
 {
-  static int rv_all = 0;
-  int i;
-
-  ds_printf("INT21: rv_all: %d + %d = ", rv_all, a);
-
-  rv_all += a;
-
-  if(rv_all > 0) {
+  static int last_all = 0;
+  if (all < 0)
+    all = last_all;
+  if (all > 0)
     memset(&vm86s.int21_revectored, 0xff, sizeof(vm86s.int21_revectored));
-  }
-  else {
-    memset(&vm86s.int21_revectored, 0x00, sizeof(vm86s.int21_revectored));
-    for(i = 0; i < 0x100; i++)
-      if(can_revector_int21(i) == REVECT) set_revectored(i, &vm86s.int21_revectored);
-  }
-
-  ds_printf("%d\n", rv_all);
+  else
+    vm86s.int21_revectored = saved_int21_revectored;
+  last_all = all;
 }
 
 
@@ -2297,7 +2109,11 @@ void int_vector_setup(void)
     if (can_revector(i)==REVECT && i!=0x21)
       set_revectored(i, &vm86s.int_revectored);
 
-  set_int21_revectored(0);
+  for (i=0; i<0x100; i++)
+    if (can_revector_int21(i)==REVECT)
+      set_revectored(i, &vm86s.int21_revectored);
+  saved_int21_revectored = vm86s.int21_revectored;
+  set_int21_revectored(-1);
 #endif
 
 #ifdef __NetBSD__
