@@ -1,3 +1,9 @@
+/* 
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ *
+ * for details see file COPYING in the DOSEMU distribution
+ */
+
 /*  DANG_BEGIN_MODULE
  *
  * REMARK
@@ -988,7 +994,10 @@ unsigned long pic_newirr;
 inline void pic_watch(s_time)
 hitimer_u *s_time;	/* time in us, 64-bit unsigned */
 {
-int timer,t_time;
+#ifndef MONOTON_MICRO_TIMING
+int timer;
+#endif
+int t_time;
 unsigned long pic_newirr;
 
   pic_newirr=pic_wirr&~pic_irr&~pic_isr;
@@ -1158,7 +1167,6 @@ void pic_sched(int ilevel, int interval)
 void pic_init(void)
 {
   /* do any one-time initialization of the PIC */
-#ifdef NEW_PORT_CODE
   emu_iodev_t  io_device;
 
   /* 8259 PIC (Programmable Interrupt Controller) */
@@ -1181,7 +1189,6 @@ void pic_init(void)
   io_device.read_portb   = read_pic1;
   io_device.write_portb  = write_pic1;
   port_register_handler(io_device, 0);
-#endif
 }
 
 void pic_reset(void)
