@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -25,11 +25,13 @@
 #include "bios.h"
 #include "kversion.h"
 #include "lpt.h"
-#include "int.h"
 
 #include "dos2linux.h"
 #include "priv.h"
 #include "utilities.h"
+#ifdef X86_EMULATOR
+#include "cpu-emu.h"
+#endif
 #include "mhpdbg.h"
 
 
@@ -108,6 +110,9 @@ config_defaults(void)
     config.rdtsc = 0;
     config.mathco = 0;
     config.smp = 0;
+#ifdef X86_EMULATOR
+    config.emuspeed = 50;	/* instruction cycles per us */
+#endif
 
     open_proc_scan("/proc/cpuinfo");
     switch (get_proc_intvalue_by_key(
@@ -150,6 +155,9 @@ config_defaults(void)
       config.realcpu, config.mathco, config.rdtsc);
 
     config.hdiskboot = 1;	/* default hard disk boot */
+#ifdef X86_EMULATOR
+    config.cpuemu = 0;
+#endif
     config.mem_size = 640;
     config.ems_size = 0;
     config.ems_frame = 0xd000;
@@ -252,8 +260,7 @@ config_defaults(void)
 				 * new VT. */
     config.debugout = NULL;	/* default to no debug output file */
 
-    config.pre_stroke =NULL;	/* default no keyboard pre-strokes */
-    config.pre_stroke_mem =NULL;
+    config.pre_stroke =NULL;	/* defualt no keyboard pre-strokes */
 
     config.sillyint = 0;
     config.must_spare_hardware_ram = 0;
@@ -283,67 +290,57 @@ config_defaults(void)
     config.mpu401_base = 0x330;
 
     config.vnet = 1;
-
-    memset(config.features, 0, sizeof(config.features));
 }
 
-static void dump_printf(char *fmt, ...)
-{
-    va_list args;
-    va_start(args, fmt);
-    vfprintf(stderr, fmt, args);
-    va_end(args);
-}
-
-void dump_config_status(void *printfunc)
+void dump_config_status(void)
 {
     char *s;
-    void (*print)(char *, ...) = (printfunc ? printfunc : dump_printf);
+    FILE *out = stderr;
 
-    if (!printfunc) {
-      (*print)("\n-------------------------------------------------------------\n");
-      (*print)("------dumping the runtime configuration _after_ parsing -----\n");
-    }
-    else (*print)("dumping the current runtime configuration:\n");
-    (*print)("Version: dosemu-" VERSTR " versioncode = 0x%08x\n\n", DOSEMU_VERSION_CODE);
-    (*print)("cpu ");
+    fprintf(out, "\n-------------------------------------------------------------\n");
+    fprintf(out, "------dumping the runtime configuration _after_ parsing -----\n");
+    fprintf(out, "Version: dosemu-" VERSTR " versioncode = 0x%08x\n\n", DOSEMU_VERSION_CODE);
+    fprintf(out, "cpu ");
     switch (vm86s.cpu_type) {
       case CPU_386: s = "386"; break;
       case CPU_486: s = "486"; break;
       case CPU_586: s = "586"; break;
       default:  s = "386"; break;
     }
-    (*print)("%s\nrealcpu ", s);
+    fprintf(out, "%s\nrealcpu ", s);
     switch (config.realcpu) {
       case CPU_386: s = "386"; break;
       case CPU_486: s = "486"; break;
       case CPU_586: s = "586"; break;
       default:  s = "386"; break;
     }
-    (*print)("%s\n", s);
-    (*print)("CPUclock %gMHz\ncpu_spd 0x%lx\ncpu_tick_spd 0x%lx\n",
+    fprintf(out, "%s\n", s);
+    fprintf(out, "CPUclock %gMHz\ncpu_spd 0x%lx\ncpu_tick_spd 0x%lx\n",
 	(((double)LLF_US)/config.cpu_spd), config.cpu_spd, config.cpu_tick_spd);
 
-    (*print)("pci %d\nrdtsc %d\nmathco %d\nsmp %d\n",
+    fprintf(out, "pci %d\nrdtsc %d\nmathco %d\nsmp %d\n",
                  config.pci, config.rdtsc, config.mathco, config.smp);
+#ifdef X86_EMULATOR
+    fprintf(out, "emuspeed %d\ncpuemu %d\n", config.emuspeed, config.cpuemu);
+#endif
 
-    (*print)("hdiskboot %d\nmem_size %d\n",
+    fprintf(out, "hdiskboot %d\nmem_size %d\n",
         config.hdiskboot, config.mem_size);
-    (*print)("ems_size 0x%x\nems_frame 0x%x\nsecure %d\n",
+    fprintf(out, "ems_size 0x%x\nems_frame 0x%x\nsecure %d\n",
         config.ems_size, config.ems_frame, config.secure);
-    (*print)("xms_size 0x%x\nmax_umb 0x%x\ndpmi 0x%x\n",
+    fprintf(out, "xms_size 0x%x\nmax_umb 0x%x\ndpmi 0x%x\n",
         config.xms_size, config.max_umb, config.dpmi);
-    (*print)("mouse_flag %d\nmapped_bios %d\nvbios_file %s\n",
+    fprintf(out, "mouse_flag %d\nmapped_bios %d\nvbios_file %s\n",
         config.mouse_flag, config.mapped_bios, (config.vbios_file ? config.vbios_file :""));
-    (*print)("vbios_copy %d\nvbios_seg 0x%x\nvbios_size 0x%x\n",
+    fprintf(out, "vbios_copy %d\nvbios_seg 0x%x\nvbios_size 0x%x\n",
         config.vbios_copy, config.vbios_seg, config.vbios_size);
-    (*print)("console %d\nconsole_keyb %d\nconsole_video %d\n",
+    fprintf(out, "console %d\nconsole_keyb %d\nconsole_video %d\n",
         config.console, config.console_keyb, config.console_video);
-    (*print)("kbd_tty %d\nexitearly %d\n",
+    fprintf(out, "kbd_tty %d\nexitearly %d\n",
         config.kbd_tty, config.exitearly);
-    (*print)("fdisks %d\nhdisks %d\nbootdisk %d\n",
+    fprintf(out, "fdisks %d\nhdisks %d\nbootdisk %d\n",
         config.fdisks, config.hdisks, config.bootdisk);
-    (*print)("term_esc_char 0x%x\nterm_color %d\nterm_updatefreq %d\n",
+    fprintf(out, "term_esc_char 0x%x\nterm_color %d\nterm_updatefreq %d\n",
         config.term_esc_char, config.term_color, config.term_updatefreq);
     switch (config.term_charset) {
       case CHARSET_LATIN: s = "latin"; break;
@@ -353,23 +350,23 @@ void dump_config_status(void *printfunc)
       case CHARSET_FULLIBM: s = "fullibm"; break;
       default: s = "(unknown)";
     }
-    (*print)("term_charset \"%s\"\nX_updatelines %d\nX_updatefreq %d\n",
+    fprintf(out, "term_charset \"%s\"\nX_updatelines %d\nX_updatefreq %d\n",
         s, config.X_updatelines, config.X_updatefreq);
-    (*print)("X_display \"%s\"\nX_title \"%s\"\nX_icon_name \"%s\"\n",
+    fprintf(out, "X_display \"%s\"\nX_title \"%s\"\nX_icon_name \"%s\"\n",
         (config.X_display ? config.X_display :""), config.X_title, config.X_icon_name);
-    (*print)("X_blinkrate %d\nX_sharecmap %d\nX_mitshm %d\n",
+    fprintf(out, "X_blinkrate %d\nX_sharecmap %d\nX_mitshm %d\n",
         config.X_blinkrate, config.X_sharecmap, config.X_mitshm);
-    (*print)("X_fixed_aspect %d\nX_aspect_43 %d\nX_lin_filt %d\n",
+    fprintf(out, "X_fixed_aspect %d\nX_aspect_43 %d\nX_lin_filt %d\n",
         config.X_fixed_aspect, config.X_aspect_43, config.X_lin_filt);
-    (*print)("X_bilin_filt %d\nX_mode13fact %d\nX_winsize_x %d\n",
+    fprintf(out, "X_bilin_filt %d\nX_mode13fact %d\nX_winsize_x %d\n",
         config.X_bilin_filt, config.X_mode13fact, config.X_winsize_x);
-    (*print)("X_winsize_y %d\nX_gamma %d\nvgaemu_memsize 0x%x\n",
+    fprintf(out, "X_winsize_y %d\nX_gamma %d\nvgaemu_memsize 0x%x\n",
         config.X_winsize_y, config.X_gamma, config.vgaemu_memsize);
-    (*print)("vesamode_list %p\nX_lfb %d\nX_pm_interface %d\n",
+    fprintf(out, "vesamode_list %p\nX_lfb %d\nX_pm_interface %d\n",
         config.vesamode_list, config.X_lfb, config.X_pm_interface);
-    (*print)("X_keycode %d\nX_font \"%s\"\nusesX %d\n",
+    fprintf(out, "X_keycode %d\nX_font \"%s\"\nusesX %d\n",
         config.X_keycode, config.X_font, config.usesX);
-    (*print)("X_mgrab_key \"%s\"\n",  config.X_mgrab_key);
+    fprintf(out, "X_mgrab_key \"%s\"\n",  config.X_mgrab_key);
     switch (config.chipset) {
       case PLAINVGA: s = "plainvga"; break;
       case TRIDENT: s = "trident"; break;
@@ -383,7 +380,7 @@ void dump_config_status(void *printfunc)
       case WDVGA: s = "wdvga"; break;
       default: s = "unknown"; break;
     }
-    (*print)("config.X %d\nhogthreshold %d\nchipset \"%s\"\n",
+    fprintf(out, "config.X %d\nhogthreshold %d\nchipset \"%s\"\n",
         config.X, config.hogthreshold, s);
     switch (config.cardtype) {
       case CARD_VGA: s = "VGA"; break;
@@ -392,9 +389,9 @@ void dump_config_status(void *printfunc)
       case CARD_EGA: s = "EGA"; break;
       default: s = "unknown"; break;
     }
-    (*print)("cardtype \"%s\"\npci_video %d\nfullrestore %d\n",
+    fprintf(out, "cardtype \"%s\"\npci_video %d\nfullrestore %d\n",
         s, config.pci_video, config.fullrestore);
-    (*print)("graphics %d\ngfxmemsize %d\nvga %d\n",
+    fprintf(out, "graphics %d\ngfxmemsize %d\nvga %d\n",
         config.graphics, config.gfxmemsize, config.vga);
     switch (config.speaker) {
       case SPKR_OFF: s = "off"; break;
@@ -402,48 +399,48 @@ void dump_config_status(void *printfunc)
       case SPKR_EMULATED: s = "emulated"; break;
       default: s = "wrong"; break;
     }
-    (*print)("dualmon %d\nforce_vt_switch %d\nspeaker \"%s\"\n",
+    fprintf(out, "dualmon %d\nforce_vt_switch %d\nspeaker \"%s\"\n",
         config.dualmon, config.force_vt_switch, s);
-    (*print)("update %d\nfreq %d\nwantdelta %d\nrealdelta %d\n",
+    fprintf(out, "update %d\nfreq %d\nwantdelta %d\nrealdelta %d\n",
         config.update, config.freq, config.wantdelta, config.realdelta);
-    (*print)("timers %d\nkeybint %d\n",
+    fprintf(out, "timers %d\nkeybint %d\n",
         config.timers, config.keybint);
-    (*print)("tty_lockdir \"%s\"\ntty_lockfile \"%s\"\nconfig.tty_lockbinary %d\n",
+    fprintf(out, "tty_lockdir \"%s\"\ntty_lockfile \"%s\"\nconfig.tty_lockbinary %d\n",
         config.tty_lockdir, config.tty_lockfile, config.tty_lockbinary);
-    (*print)("num_ser %d\nnum_lpt %d\nnum_mice %d\nfastfloppy %d\n",
+    fprintf(out, "num_ser %d\nnum_lpt %d\nnum_mice %d\nfastfloppy %d\n",
         config.num_ser, config.num_lpt, config.num_mice, config.fastfloppy);
-    (*print)("emusys \"%s\"\nemubat \"%s\"\nemuini \"%s\"\n",
+    fprintf(out, "emusys \"%s\"\nemubat \"%s\"\nemuini \"%s\"\n",
         (config.emusys ? config.emusys : ""), (config.emubat ? config.emubat : ""), (config.emuini ? config.emuini : ""));
-    (*print)("dosbanner %d\nallowvideoportaccess %d\ndetach %d\n",
+    fprintf(out, "dosbanner %d\nallowvideoportaccess %d\ndetach %d\n",
         config.dosbanner, config.allowvideoportaccess, config.detach);
-    (*print)("debugout \"%s\"\n",
+    fprintf(out, "debugout \"%s\"\n",
         (config.debugout ? config.debugout : (unsigned char *)""));
     {
 	char buf[256];
 	GetDebugFlagsHelper(buf, 0);
-	(*print)("debug_flags \"%s\"\n", buf);
+	fprintf(out, "debug_flags \"%s\"\n", buf);
     }
     {
       extern void dump_keytable(FILE *f, struct keytable_entry *kt);
-      if (!printfunc) dump_keytable(stderr, config.keytable);
+      dump_keytable(out, config.keytable);
     }
-    (*print)("pre_stroke \"%s\"\n", (config.pre_stroke ? config.pre_stroke : (unsigned char *)""));
-    (*print)("irqpassing= ");
+    fprintf(out, "pre_stroke \"%s\"\n", (config.pre_stroke ? config.pre_stroke : (unsigned char *)""));
+    fprintf(out, "irqpassing= ");
     if (config.sillyint) {
       int i;
       for (i=0; i <16; i++) {
         if (config.sillyint & (1<<i)) {
-          (*print)("IRQ%d", i);
+          fprintf(out, "IRQ%d", i);
           if (config.sillyint & (0x10000<<i))
-            (*print)("(sigio) ");
+            fprintf(out, "(sigio) ");
           else
-            (*print)(" ");
+            fprintf(out, " ");
         }
       }
-      (*print)("\n");
+      fprintf(out, "\n");
     }
-    else (*print)("none\n");
-    (*print)("must_spare_hardware_ram %d\n",
+    else fprintf(out, "none\n");
+    fprintf(out, "must_spare_hardware_ram %d\n",
         config.must_spare_hardware_ram);
     {
       int need_header_line =1;
@@ -451,15 +448,15 @@ void dump_config_status(void *printfunc)
       for (i=0; i<(sizeof(config.hardware_pages)); i++) {
         if (config.hardware_pages[i]) {
           if (need_header_line) {
-            (*print)("hardware_pages:\n");
+            fprintf(out, "hardware_pages:\n");
             need_header_line = 0;
           }
-          (*print)("%05x ", (i << 12) + 0xc8000);
+          fprintf(out, "%05x ", (i << 12) + 0xc8000);
         }
       }
-      if (!need_header_line) (*print)("\n");
+      if (!need_header_line) fprintf(out, "\n");
     }
-    (*print)("ipxsup %d\nvnet %d\npktflags 0x%x\n",
+    fprintf(out, "ipxsup %d\nvnet %d\npktflags 0x%x\n",
 	config.ipxsup, config.vnet, config.pktflags);
     
     {
@@ -467,25 +464,15 @@ void dump_config_status(void *printfunc)
 	struct printer *pptr;
 	extern struct printer lpt[NUM_PRINTERS];
 	for (i = 0, pptr = lpt; i < config.num_lpt; i++, pptr++) {
-	  (*print)("LPT%d command \"%s  %s\"  timeout %d  device \"%s\"  baseport 0x%03x\n",
+	  fprintf(out, "LPT%d command \"%s  %s\"  timeout %d  device \"%s\"  baseport 0x%03x\n",
 	  i+1, pptr->prtcmd, pptr->prtopt, pptr->delay, (pptr->dev ? pptr->dev : ""), pptr->base_port); 
 	}
     }
 
-    {
-	int i;
-	int size = sizeof(config.features) / sizeof(config.features[0]);
-	for (i = 0; i < size; i++) {
-	  (*print)("feature_%d %d\n", i, config.features[i]);
-	}
-    }
-
-    (*print)("\nSOUND:\nsb_base 0x%x\nsb_dma %d\nsb_irq %d\nmpu401_base 0x%x\nsb_dsp \"%s\"\nsb_mixer \"%s\"\n",
+    fprintf(out, "\nSOUND:\nsb_base 0x%x\nsb_dma %d\nsb_irq %d\nmpu401_base 0x%x\nsb_dsp \"%s\"\nsb_mixer \"%s\"\n",
         config.sb_base, config.sb_dma, config.sb_irq, config.mpu401_base, config.sb_dsp, config.sb_mixer);
-    if (!printfunc) {
-      (*print)("\n--------------end of runtime configuration dump -------------\n");
-      (*print)(  "-------------------------------------------------------------\n\n");
-    }
+    fprintf(out, "\n--------------end of runtime configuration dump -------------\n");
+    fprintf(out,   "-------------------------------------------------------------\n\n");
 }
 
 static void 
@@ -554,9 +541,6 @@ static void our_envs_init(char *usedoptions)
     setenv("DOSEMU_EUID", buf, 1);
     sprintf(buf, "%d", getuid());
     setenv("DOSEMU_UID", buf, 1);
-    strcpy(buf, "0");
-    if (is_console(0)) strcpy(buf, "1");
-    setenv("DOSEMU_STDIN_IS_CONSOLE", buf, 1);
 }
 
 
@@ -622,7 +606,7 @@ config_init(int argc, char **argv)
 
     opterr = 0;
     confname = CONFIG_SCRIPT;
-    while ((c = getopt(argc, argv, "ABCcF:f:I:kM:D:P:VNtsgh:H:x:KL:m23456e:E:dXY:Z:o:Ou:U:")) != EOF) {
+    while ((c = getopt(argc, argv, "ABCcF:f:I:kM:D:P:VNtsgh:H:x:KL:m23456e:E:dXY:Z:o:Ou:")) != EOF) {
 	usedoptions[(unsigned char)c] = c;
 	switch (c) {
 	case 'h':
@@ -716,11 +700,6 @@ config_init(int argc, char **argv)
 		define_config_variable(s);
 	    }
 	    break;
-	case 'U': {
-		extern void init_uhook(char *pipes);
-		init_uhook(optarg);
-	    }
-	    break;
 	}
     }
 
@@ -764,9 +743,7 @@ config_init(int argc, char **argv)
     optind = 0;
 #endif
     opterr = 0;
-    while ((c = getopt(argc, argv, "ABCcF:f:I:kM:D:P:v:VNtsgh:H:x:KLm23456e:dXY:Z:E:o:Ou:U:")) != EOF) {
-	/* currently _NOT_ used option characters: abGjJlpqQrRSTwWyz */
-
+    while ((c = getopt(argc, argv, "ABCcF:f:I:kM:D:P:v:VNtsgh:H:x:KLm23456e:dXY:Z:E:o:Ou:")) != EOF) {
 	/* Note: /etc/dosemu.conf may have disallowed some options
 	 *	 ( by removing them from $DOSEMU_OPTIONS ).
 	 *	 We skip them by re-checking 'usedoptions'.
@@ -785,7 +762,6 @@ config_init(int argc, char **argv)
 	case 'O':
 	case 'L':
 	case 'u':
-	case 'U':
 	case '2': case '3': case '4': case '5': case '6':
 	    break;
 	case 'A':
@@ -944,7 +920,7 @@ config_init(int argc, char **argv)
         geteuid(), get_cur_euid(), getegid(), get_cur_egid());
 
     if (config_check_only) {
-	dump_config_status(0);
+	dump_config_status();
 	usage();
 	leavedos(0);
     }
@@ -1003,9 +979,9 @@ int parse_debugflags(const char *s, unsigned char flag)
     char            c;
     int ret = 0;
 #ifdef X_SUPPORT
-    const char      allopts[] = "dARWDCvXkiTsm#pgcwhIExMnPrSe";
+    const char      allopts[] = "dRWDCvXkiTsm#pgcwhIExMnPrSe";
 #else
-    const char      allopts[] = "dARWDCvkiTsm#pgcwhIExMnPrSe";
+    const char      allopts[] = "dRWDCvkiTsm#pgcwhIExMnPrSe";
 #endif
 
     /*
@@ -1036,8 +1012,8 @@ int parse_debugflags(const char *s, unsigned char flag)
 	    break;
 	case 'D':		/* DOS int 21h */
 	    d.dos = flag;
-	    { static int first = 1;
-	      if(first) { set_int21_revectored(d.dos ? 1 : 0); first = 0; }
+	    { extern void set_int21_revectored(int);
+	      set_int21_revectored(d.dos != 0);
 	    }
 	    break;
         case 'C':               /* CDROM */
@@ -1058,11 +1034,8 @@ int parse_debugflags(const char *s, unsigned char flag)
 	    d.io = flag;
 	    break;
 	case 'T':		/* i/o port tracing */
-	{   extern void init_port_traceing(void);
 	    d.io_trace = flag;
-	    if (d.io_trace) init_port_traceing();
 	    break;
-	}
 	case 's':		/* serial */
 	    d.serial = flag;
 	    break;
@@ -1111,9 +1084,11 @@ int parse_debugflags(const char *s, unsigned char flag)
 	case 'S':		/* SOUND */
 	    d.sound = flag;
 	    break;
-	case 'A':		/* ASPI */
-	    d.aspi = flag;
+#ifdef X86_EMULATOR
+	case 'e':		/* cpu-emu */
+	    d.emu = flag;
 	    break;
+#endif
 	case 'a':{		/* turn all on/off depending on flag */
 		char           *newopts = (char *) malloc(strlen(allopts) + 2);
 
