@@ -1,3 +1,9 @@
+/* 
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ *
+ * for details see file COPYING in the DOSEMU distribution
+ */
+
 #include <features.h>
 #include <stdio.h>
 #include <string.h>
@@ -136,23 +142,6 @@ void stdio_init(void)
   setbuf(stdout, NULL);
 }
 
-/*
- * DANG_BEGIN_FUNCTION tmpdir_init
- *
- * description:
- *  Initialize the temporary directory
- * 
- * DANG_END_FUNCTION
- */
-void tmpdir_init(void)
-{
-  /* create tmpdir */
-  PRIV_SAVE_AREA
-  enter_priv_off();
-  mkdir(tmpdir, S_IREAD | S_IWRITE | S_IEXEC);
-  leave_priv_setting();
-}
-
 /*
  * DANG_BEGIN_FUNCTION time_setting_init
  *
@@ -163,16 +152,6 @@ void tmpdir_init(void)
  */
 void time_setting_init(void)
 {
-#ifndef NEW_CMOS
-  struct tm *tm;
-
-  time(&start_time);
-  tm = localtime((time_t *) &start_time);
-  g_printf("Set date %02d.%02d.%02d\n", tm->tm_mday, tm->tm_mon+1, tm->tm_year);
-  last_ticks = (tm->tm_hour * 60 * 60 + tm->tm_min * 60 + tm->tm_sec) * 19663/1080 /*18.206*/;
-  check_date = tm->tm_year * 10000 + tm->tm_mon * 100 + tm->tm_mday;
-  set_ticks(last_ticks);
-#endif
   initialize_timers();
 }
 
@@ -226,19 +205,15 @@ void timer_interrupt_init(void)
 void hardware_setup(void)
 {
   extern void  do_irq1(void);
-#ifdef NEW_CMOS
   extern void rtc_int8(void);
-#endif
   /* PIC init */
   pic_seti(PIC_IRQ0, timer_int_engine, 0);  /* do_irq0 in pic.c */
   pic_unmaski(PIC_IRQ0);
   pic_request(PIC_IRQ0);  /* start timer */
   pic_seti(PIC_IRQ1, do_irq1, 0); /* do_irq1 in dosio.c   */
   pic_unmaski(PIC_IRQ1);
-#ifdef NEW_CMOS
   pic_seti(PIC_IRQ8, rtc_int8, 0);
   pic_unmaski(PIC_IRQ8);
-#endif
   if (mice->intdrv || mice->type == MOUSE_PS2) {
     pic_seti(PIC_IMOUSE, DOSEMUMouseEvents, 0);
     pic_unmaski(PIC_IMOUSE);
@@ -352,37 +327,10 @@ static inline void map_hardware_ram(void)
 static inline void map_custom_bios(void)
 {
   extern void bios_f000(), bios_f000_end();
-#if 0
-  FILE   *f;
-#endif
   u_char *ptr;
 
   ptr = (u_char *) (BIOSSEG << 4);
   memcpy(ptr, bios_f000, (u_long)bios_f000_end - (u_long)bios_f000);
-
-#if 0 /* this apparently is obsolete stuff, we have all in bios.S */
-  if (!config.mapped_sbios) {
-    memset((char *) 0xffff0, 0xF4, 16);
-
-    strncpy((char *) 0xffff5, "02/25/93", 8);   /* set our BIOS date */
-    *(char *) 0xffffe = 0xfc;   /* model byte = IBM AT */
-  }
-  /* set up BIOS exit routine (we have *just* enough room for this) */
-  ptr = (u_char *) 0xffff0;
-  *ptr++ = 0xb8;                /* mov ax, 0xffff */
-  *ptr++ = 0xff;
-  *ptr++ = 0xff;
-  *ptr++ = 0xcd;                /* int DOS_HELPER_INT(0xe6) */
-  *ptr++ = DOS_HELPER_INT;
-#endif
-
-#if 0 /* for debugging only */
-  f = fopen("/tmp/bios","w");
-  fprintf(stderr,"opened /tmp/bios f=%p\n",f);
-  fwrite(ptr,0x8000,2,f);
-  fflush(f);
-  fclose(f);
-#endif
 }
 
 /* 
@@ -470,9 +418,6 @@ void memory_init(void)
 #endif
 
   bios_mem_setup();            /* setup values in BIOS area */
-#ifndef NEW_CMOS
-  cmos_reset();
-#endif
 
   /* 
    * The banner helper actually gets called *after* the VGA card
@@ -518,6 +463,7 @@ void memory_init(void)
  */
 void device_init(void)
 {
+  extern void check_console(void);
   /* check whether we are running on the console */
   check_console();
 
