@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -15,7 +15,6 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <malloc.h>
-#include <ctype.h>
 #include <string.h>
 #include <sys/ioctl.h>
 #ifdef __linux__
@@ -32,17 +31,6 @@
 #include "emu.h"
 #include "disks.h"
 #include "priv.h"
-#include "dosemu_select.h"
-#include "int.h"
-#include "fatfs.h"
-#include "utilities.h"
-
-#ifdef NEED_LLSEEK_PROTOTYPE
-  /* well, if we don't have llseek prototype,
-   * we most likely won't have __loff_t too, hence using long long
-   */
-  extern long long llseek (int fd, long long offset, int origin);
-#endif
 
 static int disks_initiated = 0;
 
@@ -80,34 +68,10 @@ static struct disk_fptr disk_fptrs[NUM_DTYPES] =
   {image_auto, image_setup},
   {hdisk_auto, hdisk_setup},
   {floppy_auto, floppy_setup},
-  {partition_auto, partition_setup},
-  {dir_auto, dir_setup}
+  {partition_auto, partition_setup}
 };
 
 
-static void dump_disk_blks(unsigned char *tb, int count, int ssiz)
-{
-  static unsigned char buf[80], cbuf[20];
-  int a,i,j;
-  unsigned char *p,*q;
-
-  q=tb; a=0;
-  while (count--) {
-    for (i=0; i < (ssiz>>4); i++) {
-      p=buf;
-      p+=sprintf(p,"%04x: ",a);
-      for (j=0; j<16; j++) {
-        p+=sprintf(p,"%02x ",*q);
-        cbuf[j] = (isprint(*q)? *q:'.');
-        q++;
-      }
-      cbuf[16]=0; a+=16;
-      d_printf("%s  %s\n",buf,cbuf);
-    }
-    d_printf("\n");
-  }
-}
-
 /* read_sectors
  *
  * okay, here's the purpose of this: to handle reads orthogonally across
@@ -142,91 +106,93 @@ read_sectors(struct disk *dp, char *buffer, long head, long sector,
 
   /* XXX - header hack. dp->header can be negative for type PARTITION */
   pos = DISK_OFFSET(dp, head, sector, track) + dp->header;
-  d_printf("DISK: %s: Trying to read %ld sectors at T/S/H %ld/%ld/%ld",
-	   dp->dev_name,count,track,sector,head);
+  d_printf("DISK: Trying to read %ld bytes at T/S/H %ld/%ld/%ld",
+	   count,track,sector,head);
 #ifdef __linux__
-  d_printf("%+Ld at pos %Ld\n", dp->header, pos);
+  d_printf(" at pos %Ld\n", pos);
 #else
   d_printf(" at pos %ld\n", pos);
 #endif
 
+
   /* reads beginning before that actual disk/file */
-  if (pos < 0 && count > 0) {
+  if (pos < 0) {
     int readsize = count * SECTOR_SIZE;
     int mbroff = -dp->header + pos;
-    int mbrread = 0;
+    int mbrcount, mbrinc;
 
-    if(!(dp->type == PARTITION || dp->type == DIR_TYPE)) {
+    if (dp->type != PARTITION) {
       error("negative offset on non-partition disk type\n");
       return -DERR_NOTFOUND;
     }
 
-    readsize = readsize > -pos ? -pos : readsize;
-
-    /* 
-     * this will end up pretending to read the requested data, but will
-     * only read as much as can be read from the fake MBR in memory
-     * and zero the rest
-     */
-
-    /* copy the MBR... */
-    if(mbroff < dp->part_info.mbr_size) {
-      mbrread = dp->part_info.mbr_size - mbroff;
-      mbrread = mbrread > readsize ? readsize : mbrread;
-      memcpy(buffer, dp->part_info.mbr + mbroff, mbrread);
-      d_printf("read 0x%lx bytes from MBR, ofs = 0x%lx (0x%lx bytes left)\n",
-        (unsigned long) mbrread, (unsigned long) mbroff, (unsigned long) (readsize - mbrread)
-      );
+    if (readsize >= -pos) {
+      mbrcount = -pos;
+      mbrinc = -pos;
+    }
+    else {
+      mbrcount = readsize;
+      mbrinc = readsize;
     }
 
-    /* ... and zero the rest */
-    if(readsize > mbrread) {
-      memset(buffer + mbrread, 0, readsize - mbrread);
-      d_printf("emulated reading 0x%lx bytes, ofs = 0x%lx\n",
-        (unsigned long) (readsize - mbrread), (unsigned long) (mbroff + mbrread)
-      );
+    /* this should take a third variable, mbr_begin, into account, but
+       * the MBR always begins at offset 0
+       * this will end up pretending to read the requested data, but will
+       * only read as much as can be read from the fake MBR in memory,
+       * simply skipping the rest.  Should I zero the "unread" parts of
+       * the dest. buffer?  No matter, I guess.
+       */
+    if ((mbroff + mbrcount) > dp->part_info.mbr_size) {
+      error("%s writing in the forbidden fake partition zone!\n",
+	    dp->dev_name);
+      mbrcount = dp->part_info.mbr_size - mbroff;
     }
 
-    if(readsize == count * SECTOR_SIZE) {
-      d_printf("   got entire read done from memory. off:%d, count:%d\n", mbroff, readsize);
+#ifdef __linux__
+    d_printf("WARNING: %Ld",pos);
+#else
+    d_printf("WARNING: %ld",pos);
+#endif
+    d_printf(" (0x%lx) read for %d below %s, "
+	     "h:%ld s:%ld t:%ld\n",
+	     (unsigned long) -pos, readsize,
+	     dp->dev_name, head, sector, track);
+
+    memcpy(buffer, dp->part_info.mbr + mbroff, mbrcount);
+
+    /* even though we may not have actually read mbrinc bytes (i.e. the
+       * read spanned non-emulated empty space), we increment this much
+       * so that we can pretend that the whole read worked and get on with
+       * the rest.
+       */
+    buffer += mbrinc;
+    pos += mbrinc;
+    already = mbrinc;
+
+    /* simulate a read() of count*SECTOR_SIZE bytes */
+    if (readsize == mbrinc) {
+      d_printf("   got entire read done from memory. off:%d, count:%d\n",
+	       mbroff, mbrcount);
+
       return readsize;
     }
-
-    buffer += readsize;
-    pos += readsize;
-    already += readsize;
   }
 
-  if(dp->type == DIR_TYPE && dp->fatfs) {
-    /* this should not never happen */
-    if(pos % SECTOR_SIZE || already % SECTOR_SIZE) {
-      error("illegal read offset for %s\n", dp->dev_name);
-      return -DERR_NOTFOUND;
-    }
-    tmpread = fatfs_read(dp->fatfs, buffer, pos / SECTOR_SIZE, count - already / SECTOR_SIZE);
-    if(tmpread == -1) return -DERR_NOTFOUND;
-    if(tmpread == -2) return -DERR_ECCERR;
-    tmpread *= SECTOR_SIZE;
-  }
-  else {
 #ifdef __linux__
-    if(pos != llseek(dp->fdesc, pos, SEEK_SET)) {
+  if (pos != llseek(dp->fdesc, pos, SEEK_SET)) {
 #else
-    if(pos != lseek(dp->fdesc, pos, SEEK_SET)) {
+  if (pos != lseek(dp->fdesc, pos, SEEK_SET)) {
 #endif
-      error("Sector not found in read_sector, error = %s!\n", strerror(errno));
-      return -DERR_NOTFOUND;
-    }
-    tmpread = RPT_SYSCALL(read(dp->fdesc, buffer, count * SECTOR_SIZE - already));
+    error("Sector not found in read_sector, error = %s!\n",
+	  strerror(errno));
+    return -DERR_NOTFOUND;
   }
 
-  if(tmpread != -1) {
-    if(d.disk > 8) dump_disk_blks(buffer, count - already / SECTOR_SIZE, SECTOR_SIZE);
-    return tmpread + already;
-  }
-  else {
+  tmpread = RPT_SYSCALL(read(dp->fdesc, buffer, count * SECTOR_SIZE));
+  if (tmpread != -1)
+    return (tmpread + already);
+  else
     return -DERR_ECCERR;
-  }
 }
 
 int
@@ -234,7 +200,7 @@ write_sectors(struct disk *dp, char *buffer, long head, long sector,
 	      long track, long count)
 {
   loff_t  pos;
-  long tmpwrite, already = 0;
+  int tmpwrite;
 
   if (dp->rdonly) {
     d_printf("ERROR: write to readonly disk %s\n", dp->dev_name);
@@ -248,61 +214,34 @@ write_sectors(struct disk *dp, char *buffer, long head, long sector,
 
   /* XXX - header hack */
   pos = DISK_OFFSET(dp, head, sector, track) + dp->header;
-  d_printf("DISK: %s: Trying to write %ld sectors T/S/H %ld/%ld/%ld",
-	   dp->dev_name,count,track,sector,head);
+  d_printf("DISK: Trying to write %ld bytes T/S/H %ld/%ld/%ld",
+	   count,track,sector,head);
 #ifdef __linux__
   d_printf(" at pos %Ld\n", pos);
 #else
   d_printf(" at pos %ld\n", pos);
 #endif
 
-  /*
-   * writes outside the partition (before the actual disk/file) are ignored
+  /* if dp->type is PARTITION, we currently don't allow writing to an area
+   * not within the actual partition (i.e. somewhere else on the faked
+   * disk). I could avoid this by somehow relocating the partition to
+   * the beginning of the disk, but I confess I'm not sure how I would
+   * do that.
+   *
+   * Also, I could allow writing to the fake MBR, but I'm not sure that's
+   * a good idea.
    */
-  if (pos < 0 && count > 0) {
-    long writesize = count * SECTOR_SIZE;
-
-    if(!(dp->type == PARTITION || dp->type == DIR_TYPE)) {
-      error("negative offset on non-partition disk type\n");
-      return -DERR_NOTFOUND;
-    }
-
-    writesize = writesize > -pos ? -pos : writesize;
-
-    d_printf("emulated writing 0x%lx bytes, ofs = 0x%lx (0x%lx left)\n",
-      writesize, (long) (pos - dp->header), count * SECTOR_SIZE - writesize
-    );
-
-    if(writesize == count * SECTOR_SIZE) return writesize;
-
-    buffer += writesize;
-    pos += writesize;
-    already += writesize;
-  }
-
-  if(dp->type == DIR_TYPE && dp->fatfs) {
-    /* this should not never happen */
-    if(pos % SECTOR_SIZE || already % SECTOR_SIZE) {
-      error("illegal write offset for %s\n", dp->dev_name);
-      return -DERR_NOTFOUND;
-    }
-    tmpwrite = fatfs_write(dp->fatfs, buffer, pos / SECTOR_SIZE, count - already / SECTOR_SIZE);
-    if(tmpwrite == -1) return -DERR_NOTFOUND;
-    if(tmpwrite == -1) return -DERR_WRITEFLT;
-    tmpwrite *= SECTOR_SIZE;
-  }
-  else {
 #ifdef __linux__
-    if(pos != llseek(dp->fdesc, pos, SEEK_SET)) {
+  if (pos != llseek(dp->fdesc, pos, SEEK_SET)) {
 #else
-    if(pos != lseek(dp->fdesc, pos, SEEK_SET)) {
+  if (pos != lseek(dp->fdesc, pos, SEEK_SET)) {
 #endif
-      error("Sector not found in write_sector!\n");
-      return -DERR_NOTFOUND;
-    }
-    tmpwrite = RPT_SYSCALL(write(dp->fdesc, buffer, count * SECTOR_SIZE - already));
+    error("Sector not found in write_sector!\n");
+    return -DERR_NOTFOUND;
   }
 
+  tmpwrite = RPT_SYSCALL(write(dp->fdesc, buffer, count * SECTOR_SIZE));
+
   /* this should make floppies a little safer...I would as soon use the
    * O_SYNC flag, but we don't have it.  fsync() should be in the kernel
    * soon, I think, thanks to Stephen Tweedie.
@@ -312,7 +251,7 @@ write_sectors(struct disk *dp, char *buffer, long head, long sector,
 
   FLUSHDISK(dp);
 
-  return tmpwrite + already;
+  return tmpwrite;
 }
 
 void
@@ -408,90 +347,6 @@ hdisk_auto(struct disk *dp)
 #endif
 }
 
-void dir_auto(struct disk *dp)
-{
-  /*
-   * We emulate an entire disk with 1 partition starting at t/h/s 0/1/1.
-   * You are free to change the geometry (e.g. to change the partition size).
-   */
-  dp->sectors = 63;
-  dp->heads = 255;
-  dp->tracks = 50;
-
-  dp->start = dp->sectors;
-
-  d_printf(
-    "DIR auto_info disk %s; h=%d, s=%d, t=%d, start=%ld\n",
-    dp->dev_name, dp->heads, dp->sectors, dp->tracks, dp->start
-  );
-}
-
-void dir_setup(struct disk *dp)
-{
-  unsigned char *mbr;
-  struct partition *pi = &dp->part_info;
-  int i = strlen(dp->dev_name);
-
-  while(--i >= 0) if(dp->dev_name[i] == '/') dp->dev_name[i] = 0; else break;
-
-  d_printf("partition setup for directory %s\n", dp->dev_name);
-
-  pi->beg_head = 1;
-  pi->beg_sec = 1;
-  pi->beg_cyl = 0;
-  pi->end_head = dp->heads - 1;
-  pi->end_sec = dp->sectors;
-  pi->end_cyl = dp->tracks - 1;
-  pi->pre_secs = dp->sectors;
-  pi->num_secs = (dp->tracks * dp->heads - 1) * dp->sectors;
-
-  dp->header = -(SECTOR_SIZE * (loff_t) (pi->pre_secs));
-
-  pi->mbr_size = SECTOR_SIZE;
-  pi->mbr = malloc(pi->mbr_size);
-  mbr = pi->mbr;
-
-  memset(mbr, 0, SECTOR_SIZE);
-  /*
-   * mov ax,0fffeh
-   * int 0e6h
-   */
-  mbr[0x00] = 0xb8;
-  mbr[0x01] = 0xfe;
-  mbr[0x02] = 0xff;
-  mbr[0x03] = 0xcd;
-  mbr[0x04] = 0xe6;
-  mbr[PART_INFO_START + 0x00] = PART_BOOT;
-  mbr[PART_INFO_START + 0x01] = pi->beg_head;
-  mbr[PART_INFO_START + 0x02] = pi->beg_sec + ((pi->beg_cyl & 0x300) >> 2);
-  mbr[PART_INFO_START + 0x03] = pi->beg_cyl;
-  mbr[PART_INFO_START + 0x04] = pi->num_secs < 1 << 16 ? 0x04 : 0x06;
-  mbr[PART_INFO_START + 0x05] = pi->end_head;
-  mbr[PART_INFO_START + 0x06] = pi->end_sec + ((pi->end_cyl & 0x300) >> 2);
-  mbr[PART_INFO_START + 0x07] = pi->end_cyl;
-  *(unsigned *) (mbr + PART_INFO_START + 0x08) = pi->pre_secs;
-  *(unsigned *) (mbr + PART_INFO_START + 0x0c) = pi->num_secs;
-  mbr[SECTOR_SIZE - 2] = 0x55;
-  mbr[SECTOR_SIZE - 1] = 0xaa;
-
-  d_printf("partition table entry for device %s is:\n", dp->dev_name);
-  d_printf(
-    "beg head %d, sec %d, cyl %d = end head %d, sec %d, cyl %d\n",
-    pi->beg_head, pi->beg_sec, pi->beg_cyl,
-    pi->end_head, pi->end_sec, pi->end_cyl
-  );
-  d_printf(
-    "pre_secs %ld, num_secs %ld = %lx, -dp->header %ld = 0x%lx\n",
-    pi->pre_secs, pi->num_secs, pi->num_secs,
-    (long) -dp->header, (unsigned long) -dp->header
-  );
-
-  dp->fatfs = NULL;
-  fatfs_init(dp);
-
-  if(dp->fatfs && !redir_state) set_int21_revectored(redir_state = 1);
-}
-
 /* XXX - relies upon a file of SECTOR_SIZE in PARTITION_PATH that which
  *       contains the MBR (first sector) of the drive (i.e. /dev/hda).
  *       only works for /dev/hda1 right now, and only allows one
@@ -531,10 +386,9 @@ partition_setup(struct disk *dp)
 #endif
 
   enter_priv_on();
-  part_fd = SILENT_DOS_SYSCALL(open(hd_name, O_RDONLY));
+  part_fd = DOS_SYSCALL(open(hd_name, O_RDONLY));
   leave_priv_setting();
   if (part_fd == -1) {
-    if (dp->removeable) return;
     error("opening device %s to read MBR for PARTITION %s\n",
 	  hd_name, dp->dev_name);
     leavedos(22);
@@ -671,9 +525,6 @@ d_nullf(struct disk *dp)
   d_printf("NULLF for %s\n", dp->dev_name);
 }
 
-
-unsigned char ATAPI_buf0[512] = { 0 };
-
 inline void
 disk_close(void)
 {
@@ -682,14 +533,13 @@ disk_close(void)
   if (!disks_initiated) return;  /* just to be safe */
   for (dp = disktab; dp < &disktab[FDISKS]; dp++) {
     if (dp->removeable && dp->fdesc >= 0) {
-      d_printf("DISK: %s: Closing a disk\n",dp->dev_name);
+      d_printf("DISK: Closing a disk\n");
       (void) close(dp->fdesc);
       dp->fdesc = -1;
     }
   }
 }
 
-
 #ifdef __NetBSD__
 void
 disk_open(struct disk *dp)
@@ -750,6 +600,18 @@ disk_open(struct disk *dp)
 }
 #endif
 
+static void sigalarm_onoff(int on)
+{
+  static struct itimerval itv_old;
+  static struct itimerval itv;
+  if (on) setitimer(TIMER_TIME, &itv_old, NULL);
+  else {
+    itv.it_interval.tv_sec = itv.it_interval.tv_usec = 0;
+    itv.it_value = itv.it_interval;
+    setitimer(TIMER_TIME, &itv, &itv_old);
+  }
+}
+
 #ifdef __linux__
 void
 disk_open(struct disk *dp)
@@ -761,15 +623,10 @@ disk_open(struct disk *dp)
     return;
     
   enter_priv_on();
-  dp->fdesc = SILENT_DOS_SYSCALL(open(dp->type == DIR_TYPE ? "/dev/null" : dp->dev_name, dp->wantrdonly ? O_RDONLY : O_RDWR, 0));
+  dp->fdesc = SILENT_DOS_SYSCALL(open(dp->dev_name, dp->wantrdonly ? O_RDONLY : O_RDWR, 0));
   leave_priv_setting();
 
-  /* FIXME:
-   * Why the hell was the below handling restricted to non-removeable disks?
-   * This made opening writeprotected floppies impossible :-(
-   *                                                  -- Hans, 990112
-   */
-  if ( /*!dp->removeable &&*/ (dp->fdesc < 0)) {
+  if (dp->fdesc < 0) {
     if (errno == EROFS || errno == ENODEV) {
       enter_priv_on();
       dp->fdesc = DOS_SYSCALL(open(dp->dev_name, O_RDONLY, 0));
@@ -803,33 +660,20 @@ disk_open(struct disk *dp)
    * To overcome this problem we temporary switch off the timer
    * during the ioctl. (well, not what we really like)
    * ( 19 May 1996, Hans Lermen ) */
-  int res=0;
-  if (dp->default_cmos == ATAPI_FLOPPY) {
-	unsigned long tns;
-	if (ATAPI_buf0[0] || (read_sectors(dp, ATAPI_buf0, 0, 0, 0, 1) > 0)) {
-	      fl.sect = *((unsigned char *)&ATAPI_buf0[0x18]);
-	      fl.head = *((unsigned char *)&ATAPI_buf0[0x1a]);
-	      tns = *((unsigned short *)&ATAPI_buf0[0x13]);
-	      if (tns==0) tns = *((unsigned long *)&ATAPI_buf0[0x20]);
-	      fl.track = tns/(fl.sect*fl.head);
-	}
-	else {	/* no disk available */
-	      dp->sectors = 0;
-	      dp->heads = 0;
-	      dp->tracks = 0;
-	      return;
-	}
-  }
-  else {
+  int res;
+ #ifdef USE_THREADS
+  static int background_ioctl(int fd, int request, void *param);
+  res = background_ioctl(dp->fdesc, FDGETPRM, &fl);
+ #else
   sigalarm_onoff(0);
   res = ioctl(dp->fdesc, FDGETPRM, &fl);
   sigalarm_onoff(1);
-  }
+ #endif
   if (res == -1) {
 #else
   if (ioctl(dp->fdesc, FDGETPRM, &fl) == -1) {
 #endif
-    if ((dp->fdesc == -1) || (errno == ENODEV)) {	/* no disk available */
+    if (errno == ENODEV) {	/* no disk available */
       dp->sectors = 0;
       dp->heads = 0;
       dp->tracks = 0;
@@ -844,8 +688,7 @@ disk_open(struct disk *dp)
   dp->sectors = fl.sect;
   dp->heads = fl.head;
   dp->tracks = fl.track;
-  if (dp->default_cmos != ATAPI_FLOPPY)
-    DOS_SYSCALL(ioctl(dp->fdesc, FDMSGOFF, 0));
+  DOS_SYSCALL(ioctl(dp->fdesc, FDMSGOFF, 0));
 }
 #endif
 
@@ -862,7 +705,6 @@ disk_close_all(void)
     d_printf("BOOTDISK Closing\n");
   }
   for (dp = disktab; dp < &disktab[FDISKS]; dp++) {
-    ATAPI_buf0[0] = 0;
     if (dp->fdesc >= 0) {
       d_printf("Floppy disk Closing %x\n", dp->fdesc);
       (void) close(dp->fdesc);
@@ -870,7 +712,6 @@ disk_close_all(void)
     }
   }
   for (dp = hdisktab; dp < &hdisktab[HDISKS]; dp++) {
-    if(dp->type == DIR_TYPE) fatfs_done(dp);
     if (dp->fdesc >= 0) {
       d_printf("Hard disk Closing %x\n", dp->fdesc);
       (void) close(dp->fdesc);
@@ -920,28 +761,23 @@ disk_init(void)
           d_printf("(disk) can't open bootdisk %s for read/write. Readonly did work though\n", bootdisk.dev_name);
         }
       } else {
-        error("can't open bootdisk %s: %s\n", dp->dev_name, strerror(errno));
+        error("can't open bootdisk %s: %sn", dp->dev_name, strerror(errno));
         leavedos(23);
       }
     }
     else bootdisk.rdonly = bootdisk.wantrdonly;
     bootdisk.removeable = 0;
   }
-
-  /*
-   * Open floppy disks
-   */
-  ATAPI_buf0[0] = 0;
   for (dp = disktab; dp < &disktab[FDISKS]; dp++) {
     if (stat(dp->dev_name, &stbuf) < 0) {
       error("can't stat %s\n", dp->dev_name);
       leavedos(24);
     }
     if (S_ISBLK(stbuf.st_mode))
-      d_printf("ISBLK ");
+      d_printf("ISBLK\n");
     if (S_ISCHR(stbuf.st_mode))
-      d_printf("ISCHR ");
-    d_printf("dev %s: %#x\n", dp->dev_name, (unsigned) stbuf.st_rdev);
+      d_printf("ISCHR\n");
+    d_printf("dev : %x\n", stbuf.st_rdev);
 #ifdef __NetBSD__
     if ((S_ISBLK(stbuf.st_mode) && major(stbuf.st_rdev) == 0x2) ||
 	(S_ISCHR(stbuf.st_mode) && major(stbuf.st_rdev) == 0x9)) {
@@ -952,9 +788,7 @@ disk_init(void)
     }
 #endif
 #ifdef __linux__
-    if (S_ISBLK(stbuf.st_mode) && 
-    (((stbuf.st_rdev & 0xff00)==0x200) || (dp->default_cmos==ATAPI_FLOPPY))
-    ) {
+    if (S_ISBLK(stbuf.st_mode) && (stbuf.st_rdev & 0xff00) == 0x200) {
       d_printf("DISK %s removeable\n", dp->dev_name);
       dp->removeable = 1;
       dp->fdesc = -1;
@@ -978,16 +812,11 @@ disk_init(void)
         }
       } else {
         error("can't open %s: %s\n", dp->dev_name, strerror(errno));
-        /* 'leavedos' here hangs the machine */
         leavedos(25);
       }
     }
     else dp->rdonly = dp->wantrdonly;
   }
-
-  /*
-   * Open hard disks
-   */
   for (dp = hdisktab; dp < &hdisktab[HDISKS]; dp++) {
     if(dp->type == IMAGE)  {
 	if (dp->dexeflags & DISK_DEXE_RDWR) {
@@ -1000,7 +829,7 @@ disk_init(void)
 	}
     }
     else enter_priv_on();
-    dp->fdesc = open(dp->type == DIR_TYPE ? "/dev/null" : dp->dev_name, dp->rdonly ? O_RDONLY : O_RDWR, 0);
+    dp->fdesc = open(dp->dev_name, dp->rdonly ? O_RDONLY : O_RDWR, 0);
     leave_priv_setting();
     if (dp->fdesc < 0) {
       if (errno == EROFS || errno == EACCES) {
@@ -1083,11 +912,11 @@ int
 checkdp(struct disk *disk)
 {
   if (disk == NULL) {
-    d_printf("DISK: null dp\n");
+    error("DISK: null dp\n");
     return 1;
   }
   else if (disk->fdesc == -1) {
-    d_printf("DISK: closed disk\n");
+    error("DISK: closed disk\n");
     return 1;
   }
   else
@@ -1164,7 +993,7 @@ int13(u_char i)
 
     if (checkdp_val || head >= dp->heads ||
 	sect >= dp->sectors || track >= dp->tracks) {
-      d_printf("Sector not found 1!\n");
+      error("Sector not found 1!\n");
       d_printf("DISK %d read [h:%d,s:%d,t:%d](%d)->%p\n",
 	       disk, head, sect, track, number, (void *) buffer);
       if (dp) {
@@ -1228,7 +1057,7 @@ int13(u_char i)
     }
 
     if (dp->rdonly) {
-      W_printf("write protect!\n");
+      error("write protect!\n");
       show_regs(__FILE__, __LINE__);
       if (dp->removeable)
 	HI(ax) = DERR_WP;
@@ -1239,7 +1068,7 @@ int13(u_char i)
     }
 
     if (dp->rdonly)
-      W_printf("CONTINUED!!!!!\n");
+      error("CONTINUED!!!!!\n");
     res = write_sectors(dp, buffer, head, sect, track, number);
 
     if (res < 0) {
@@ -1313,32 +1142,33 @@ int13(u_char i)
     break;
 
   case 8:			/* get disk drive parameters */
-    d_printf("disk get parameters %#x\n", disk);
+    d_printf("disk get parameters %d\n", disk);
 
     if (dp != NULL) {
       /* get CMOS type */
+      /* LO(bx) = 3; */
       switch (dp->sectors) {
       case 9:
-	LO(bx) = THREE_INCH_720KFLOP;
+	LO(bx) = 3;
 	break;
       case 15:
-	LO(bx) = FIVE_INCH_FLOPPY;
+	LO(bx) = 2;
 	break;
       case 18:
-	LO(bx) = THREE_INCH_FLOPPY;
+	LO(bx) = 4;
 	break;
       case 0:
 	LO(bx) = dp->default_cmos;
 	dp->tracks = 80;
 	dp->heads = 2;
-	if (LO(bx) == FIVE_INCH_FLOPPY)
-	  dp->sectors = 15;
-	else
+	if (LO(bx) == 4)
 	  dp->sectors = 18;
+	else
+	  dp->sectors = 15;
 	d_printf("auto type defaulted to CMOS %d, sectors: %d\n", LO(bx), dp->sectors);
 	break;
       default:
-	LO(bx) = THREE_INCH_FLOPPY;
+	LO(bx) = 4;
 	d_printf("type det. failed. num_tracks is: %d\n", dp->tracks);
 	break;
       }
@@ -1348,7 +1178,7 @@ int13(u_char i)
       HI(cx) = (dp->tracks - 1) & 0xff;
 
       LO(dx) = (disk < 0x80) ? FDISKS : HDISKS;
-      LO(cx) = (dp->sectors & 0x3f) | (((dp->tracks -1) & 0x300) >> 2);
+      LO(cx) = dp->sectors | (((dp->tracks -1) & 0x300) >> 2);
       LO(ax) = 0;
       HI(ax) = DERR_NOERR;
       REG(eflags) &= ~CF;	/* no error */
@@ -1418,7 +1248,7 @@ int13(u_char i)
     /* end of Alan's additions */
 
   case 0x15:			/* Get type */
-    d_printf("disk gettype %#x\n", disk);
+    d_printf("disk gettype %d\n", disk);
     if (dp != NULL && disk >= 0x80) {
       if (dp->removeable) {
 	HI(ax) = 1;		/* floppy disk, change detect (1=no, 2=yes) */
@@ -1679,7 +1509,7 @@ int13(u_char i)
     NOCARRY;
     break;
   default:
-    d_printf("disk error, unknown command: int13, ax=0x%x\n",
+    error("disk error, unknown command: int13, ax=0x%x\n",
 	  LWORD(eax));
     show_regs(__FILE__, __LINE__);
     CARRY;
@@ -1703,3 +1533,48 @@ floppy_tick(void)
   }
 }
 
+#ifdef USE_THREADS
+
+#include "lt-threads.h"
+
+static mbox_handle ioctlmbox_out;
+static mbox_handle ioctlmbox_in;
+
+struct {
+  struct msg msg;
+  int fd;
+  int request;
+  void *param;
+  int result;
+} ioctlmsg;
+
+void background_ioctl_thread(int start)
+{
+  if (!start) {
+    /* we come here _before_ any thread is started
+     * but threading is active and we run at scope of thread0.
+     * We now create the resource we need.
+     */
+     ioctlmbox_in = create_local_mailbox("diskioctl_in", 2);
+     ioctlmbox_out = create_local_mailbox("diskioctl_out", 2);
+     return;
+  }
+  /* here we are a separate thread */
+  while (1) {
+    receivemessage(ioctlmbox_in);
+    ioctlmsg.result = ioctl(ioctlmsg.fd, ioctlmsg.request, ioctlmsg.param);
+    sendmessage(ioctlmbox_out,0);
+  }
+}
+
+static int background_ioctl(int fd, int request, void *param)
+{
+  ioctlmsg.fd = fd;
+  ioctlmsg.request = request;
+  ioctlmsg.param = param;
+  sendmessage(ioctlmbox_in,0);
+  receivemessage(ioctlmbox_out);
+  return ioctlmsg.result;
+}
+
+#endif
