@@ -1,16 +1,14 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
 
 #include <stdlib.h>
 #include <unistd.h>
-#if GLIBC_VERSION_CODE >= 2000
-  #include <sys/io.h>
-#endif
 #include "emu.h"
 #include "priv.h"
+#include "lt-threads.h"
 
 #if 0
 #define PRIV_TESTING
@@ -19,8 +17,15 @@
 /* Some handy information to have around */
 static uid_t uid,euid;
 static gid_t gid,egid;
+#ifndef USE_THREADS
 static uid_t cur_uid, cur_euid;
 static gid_t cur_gid, cur_egid;
+#else
+#define cur_uid  (OWN_TCB->uid)
+#define cur_euid (OWN_TCB->euid)
+#define cur_gid  (OWN_TCB->gid)
+#define cur_egid (OWN_TCB->egid)
+#endif
 
 static int skip_priv_setting = 0;
 
