@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -27,6 +27,9 @@
 #include "pic.h"
 #include "dpmi.h"
 #include "utilities.h"
+#ifdef USE_THREADS
+#include "lt-threads.h"
+#endif
 #ifdef USE_MHPDBG
 #include "mhpdbg.h"
 #endif
@@ -64,8 +67,14 @@ static char *timestamp (char *p)
   unsigned long t;
   int i;
 
+#ifdef X86_EMULATOR
+  if ((config.cpuemu>1) && in_vm86) t=0; else
+#endif
   t = GETusTIME(0)/1000;
   /* [12345678]s - SYS time */
+#ifdef X86_EMULATOR
+    if ((config.cpuemu>1) && in_vm86) strcpy(p,"[********] "); else
+#endif
     {
       p[0] = '[';
       for (i=8; i>0; --i)
@@ -147,6 +156,9 @@ int vlog_printf(int flg, const char *fmt, va_list args)
 #endif
      ) return 0;
 
+#ifdef USE_THREADS
+  lock_resource(resource_libc);
+#endif
   {
     char *q;
 
@@ -177,6 +189,9 @@ int vlog_printf(int flg, const char *fmt, va_list args)
       }
     }
   }
+#ifdef USE_THREADS
+  unlock_resource(resource_libc);
+#endif
   return i;
 }
 
@@ -203,8 +218,7 @@ void verror(const char *fmt, va_list args)
 		vfprintf(stderr, fmt+1, args);
 	}
 	else {
-		fmtbuf[sizeof(fmtbuf)-1] = 0;
-		snprintf(fmtbuf, sizeof(fmtbuf)-1, "ERROR: %s", fmt);
+		sprintf(fmtbuf, "ERROR: %s", fmt);
 		vlog_printf(10, fmtbuf, args);
 		vfprintf(stderr, fmtbuf, args);
 	}
@@ -229,8 +243,7 @@ p_dos_str(char *fmt,...) {
   int i;
 
   va_start(args, fmt);
-  buf[sizeof(buf)-1] = 0;
-  i = vsnprintf(buf, sizeof(buf)-1,  fmt, args);
+  i = vsprintf(buf, fmt, args);
   va_end(args);
 
   s = buf;
@@ -418,68 +431,3 @@ char *get_dosemu_local_home(void)
 {
 	return mkdir_under(get_path_in_HOME(".dosemu"), 0, 0);
 }
-
-#if (GLIBC_VERSION_CODE >= 2000) && defined(PORTABLE_BINARY)
-/*
- * Under glibc we can't make portable static binaries due to libnss stuff.
- * As we only need to look at /etc/passwd (and which non-libnss system won't
- * have it?) we work around this problem by doing getpwuid() getpwnam()
- * ourselves, if we have no /etc/nsswitch.conf.
- *                                                -- Hans, 981201
- */
-
-static struct passwd *our_getpw(const char *name, uid_t uid)
-{
-  FILE *f;
-  static struct passwd pw;
-  static char buf[1024];
-  char *p;
-
-  f = fopen("/etc/passwd", "r");
-  if (!f) return 0;
-  buf[sizeof(buf)-1] = 0;
-  while (fgets(buf, sizeof(buf)-1, f)) {
-    pw.pw_name = strtok(buf, ":");
-    if (!pw.pw_name) continue;
-    if (name && strcmp(name, pw.pw_name)) continue;
-    pw.pw_passwd = strtok(0, ":");
-    if (!pw.pw_passwd) continue;
-    p = strtok(0, ":");
-    if (!p) continue;
-    pw.pw_uid = (uid_t)strtoul(p,0,0);
-    if (!name && pw.pw_uid != uid) continue;
-    p = strtok(0, ":");
-    if (!p) continue;
-    pw.pw_gid = (gid_t)strtoul(p,0,0);
-    /* we don't need more, hence we skip the rest */
-    fclose(f);
-    return &pw;
-  }
-  fclose(f);
-  return 0;
-}
-
-static int have_libnss()
-{
-  static int have_it = -1;
-  if (have_it >= 0 ) return have_it;
-  have_it = exists_file("/etc/nsswitch.conf") ? 1: 0;
-  return have_it;
-}
-
-#undef getpwuid
-#undef getpwnam
-
-struct passwd *our_getpwuid(uid_t uid)
-{
-  if (have_libnss()) return getpwuid(uid);
-  return our_getpw(0, uid);
-}
-
-struct passwd *our_getpwnam(const char *name)
-{
-  if (have_libnss()) return getpwnam(name);
-  return our_getpw(name, 0);
-}
-
-#endif /* GLIBC_VERSION_CODE >= 2000 */
