@@ -1,3 +1,10 @@
+/* 
+ * All modifications in this file to the original code are
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ *
+ * for details see file COPYING in the DOSEMU distribution
+ */
+
 /* DANG_BEGIN_MODULE
  * 
  * REMARK
@@ -323,9 +330,7 @@ com_writeb(ioport_t port, Bit8u value) {
  */
 static void do_ser_init(int num)
 {
-#ifdef NEW_PORT_CODE
   emu_iodev_t io_device;
-#endif
   int data = 0;
   int i;
   
@@ -395,7 +400,6 @@ static void do_ser_init(int num)
 
   /*** The following is where the real initialization begins ***/
 
-#ifdef NEW_PORT_CODE
   /* Tell the port manager that we exist and that we're alive */
   io_device.read_portw  = NULL;
   io_device.write_portw = NULL;
@@ -428,7 +432,6 @@ static void do_ser_init(int num)
       break;
   }
   port_register_handler(io_device, 0);
-#endif
 
   /* Information about serial port added to debug file */
   s_printf("SER%d: COM%d, intlevel=%d, base=0x%x, device=%s\n", 
