@@ -1,5 +1,11 @@
 #!/usr/bin/perl
 #
+
+if ($0 =~ /DANG_c\.pl/) {$dang_mode = 1;}
+else {$dang_mode = 0;}
+
+# if (dang_mode) {
+#
 # DANG Compiler
 # -------------
 #
@@ -36,6 +42,35 @@
 # <NOTE: This doesn't use DANG markers, since it is a) not part of DOSEMU and
 #  b) the markers at the start would confuse things ...!>
 #
+# }
+#
+# if (!dang_mode) {
+#
+# NOTE: This is retrieved from the work of Alistair MacDonald
+#       and (though adapted to general purpose) remains
+#       (C) Alistair MacDonald <alistair@slitesys.demon.co.uk> 
+#
+#       The original software is named DANG (Dosemu Alterer Novice Guide) and
+#       as such is part of the DOSEMU distribution.
+#
+# SIDOC (Source Imbedded DOCumentation) Compiler
+# ---------------------------------------------
+#
+# Converts the comments in the source code into items suitable for
+# inclusion in the SIDOC.
+#
+# The Markers are :
+#
+# SIDOC_BEGIN_MODULE    / SIDOC_END_MODULE
+# SIDOC_BEGIN_CHANGELOG / SIDOC_END_CHANGELOG
+# SIDOC_BEGIN_REMARK    / SIDOC_END_REMARK
+# SIDOC_BEGIN_FUNCTION  / SIDOC_END_FUNCTION
+# SIDOC_BEGIN_NEWIDEA   / SIDOC_END_NEWIDEA
+# SIDOC_FIXTHIS
+#
+# }
+#
+#
 # Command line options 
 # --------------------
 # -v           - verbose. 
@@ -57,6 +92,10 @@ require "getopts.pl";
     undef @intro;
     undef @finally;
 
+    $main_title = "";
+    $main_author = "";
+    $main_abstract = "";
+
     if ($opt_c !~ /.+/) {
 	die "$0: The option -c takes one compulsory argument - a configuration filename.\n";
     } elsif (defined $opt_c) {
@@ -69,21 +108,22 @@ require "getopts.pl";
 	&build_config (undef);
 	close INPUT;
 
-	&get_version ;
+	if ($dang_mode) {&get_dosemu_version;}
+	else {&get_version;}
 	
-	$OUTPUT = ">>DANG.sgml";
+	if ($dang_mode) {$OUTPUT = ">>DANG.sgml";}
+	else {$OUTPUT = ">>SIDOC.sgml";}
 	open OUTPUT;
 
 	print OUTPUT <<"EndOfPreamble";
 <!doctype linuxdoc system>
 <article>
 
-<title>DANG
-The DOSEMU Alterer Novices Guide
-<author>Alistair MacDonald, <tt/alistair\@slitesys.demon.co.uk/
-<date>For DOSEMU $version
+<title> $main_title
+<author> Author: $main_author
+<date> version $version
 <abstract>
-This Document is the DOSEMU Alterer Novices Guide. It is known as the DANG.
+$main_abstract
 </abstract>
 
 <toc>
@@ -201,36 +241,47 @@ EndOfPreamble
     close OUTPUT;
 }			       
 
-
-
 # Sets $version to be the Version of DOSEMU used to generate the source.
 # This is grabbed from dosconfig.h
 
+sub get_dosemu_version  {
+    local (*INPUT);
+    local ($dat);
+    $INPUT = $base_dir. "../VERSION";
+    open INPUT;
+    $dat = <INPUT>;
+    chop $dat;
+    $version = "dosemu-$dat";
+    close INPUT;
+}
+
+# Sets $version to be the Version of the source paket.
+# This is grabbed from main Makefile
+
 sub get_version  {
     local (*INPUT);
-    local ($emupl, $emuver);
+    local ($pl, $ver, $sublv);
 
-    $INPUT = $base_dir. "../configure.in";
+    $INPUT = $base_dir. "Makefile";
     open INPUT;
 
     while (<INPUT>) {
         if (/PATCHLEVEL="(.*)"/) {
-	    $emupl = $1;
+	    $pl = $1;
 	    next;
         }        
         if (/^VERSION="(.*)"/) {
-	    $emuver = $1;
+	    $ver = $1;
         }        
         if (/^SUBLEVEL="(.*)"/) {
-	    $emusublevel = $1;
+	    $sublv = $1;
         }        
     }
 
-    $version = "v$emuver.$emusublevel pl$emupl";
+    $version = "v$ver.$sublv.$pl";
     close INPUT;
 }
 
-
 # Yup - Its another one of these pattern matchers .... 8-(
 #
 
@@ -241,7 +292,7 @@ sub build_config {
     # '#' signify comments - any extra character will be deleted.
     # GROUP: <name> <summary> <file1> <file2> ... <filen>
     #   This may be spread over multiple lines ... Groups are related files
-    #   ie the mouse files, the video files. They are related in DANG too ...
+    #   ie the mouse files, the video files. They are related in DANG/SIDOC too ...
     # SUMMARY: <summary>
     #   The master summary file.
     # BASEDIR: directory
@@ -264,6 +315,21 @@ sub build_config {
 
     chop;
 
+    if (/^\s*TITLE:\s*/) {
+        $main_title = "$main_title" . $';
+	return;
+    }
+
+    if (/^\s*AUTHOR:\s*/) {
+        $main_author = "$main_author" . $';
+	return;
+    }
+
+    if (/^\s*ABSTRACT:\s*/) {
+        $main_abstract = "$main_abstract" . $';
+	return;
+    }
+
     if (/^\s*GROUP:\s*/) {
 	if ($group) {
 	    push (@groups, $data);
@@ -340,6 +406,35 @@ sub protect_specials {
 }
 
 
+# This back-converts any special characters in the output into the relevant
+# protected items ....
+
+sub un_protect_specials {
+    local ($text) = @_;
+    local ($save);
+
+    $save = $*;
+    $* = 1;
+
+    $text =~ s/&rsqb;/\]/g;
+
+    $text =~ s/&lsqb;/\[/g;
+    $text =~ s/&dquot;/"/g;     # " (For emacs !)
+    $text =~ s/&percnt;/%/g;
+    $text =~ s/&num;/\#/g;
+    $text =~ s/&dollar;/\$/g;
+    $text =~ s/&gt;/>/g;
+    $text =~ s/&lt;/</g;
+    $text =~ s/&etago;/\<\\/g;
+    $text =~ s/&amp;/&/g;
+
+
+    $* = $save;
+
+    return $text;
+}
+
+
 #
 # This is the routine which does all the scanning. 
 # The scanning information is only mentioned here. This program is not intended
@@ -359,6 +454,7 @@ sub scan_file  {
 
     undef @fixthis;		# Just to make sure ...!
     undef @functions;
+    undef @structures;
     undef @modules;
     undef @newideas;
     undef @changes;
@@ -380,7 +476,7 @@ sub scan_file  {
     {
 	$line ++;
 
-	if (/DANG_BEGIN_MODULE/) {
+	if (/(SIDOC|DANG)_BEGIN_MODULE/) {
 	    # Module Start Found ...
 	    if ($module != 0) {
 		print "Already Found a Module start at line $start_line. Ignoring.\n";
@@ -401,7 +497,7 @@ sub scan_file  {
 	    next;
 	}
 
-	if (/DANG_END_MODULE/) {
+	if (/(SIDOC|DANG)_END_MODULE/) {
 	    # Module End Found.
 	    if ($module != 1) {
 		print "No Module Start found (END at line $line). Ignoring.\n";
@@ -419,7 +515,7 @@ sub scan_file  {
 	    next;
 	}
 
-	if (/DANG_BEGIN_CHANGELOG/) {
+	if (/(SIDOC|DANG)_BEGIN_CHANGELOG/) {
 	    # Changelog Start Found ...
 	    if ($changes != 0) {
 		print "Already Found a Changelog start at line $start_line. Ignoring.\n";
@@ -440,7 +536,7 @@ sub scan_file  {
 	    next;
 	}
 
-	if (/DANG_END_CHANGELOG/) {
+	if (/(SIDOC|DANG)_END_CHANGELOG/) {
 	    # Changelog End Found.
 	    if ($changes != 1) {
 		print "No Changelog Start found (END at line $line). Ignoring.\n";
@@ -458,7 +554,7 @@ sub scan_file  {
 	    next;
 	}
 
-	if (/DANG_BEGIN_FUNCTION/) {
+	if (/(SIDOC|DANG)_BEGIN_FUNCTION/) {
 	    # Function Start Found ...
 	    if ($functions != 0) {
 		print "Already Found a Function start at line $start_line. Ignoring.\n";
@@ -479,7 +575,7 @@ sub scan_file  {
 	    next;
 	}
 
-	if (/DANG_END_FUNCTION/) {
+	if (/(SIDOC|DANG)_END_FUNCTION/) {
 	    # Function End Found.
 	    if ($functions != 1) {
 		print "No Function Start found (END at line $line). Ignoring.\n";
@@ -497,7 +593,46 @@ sub scan_file  {
 	    next;
 	}
 
-	if (/DANG_BEGIN_REMARK/) {
+	if (/(SIDOC|DANG)_BEGIN_STRUCT/) {
+	    # Structure Start Found ...
+	    if ($structures != 0) {
+		print "Already Found a Structure start at line $start_line. Ignoring.\n";
+	    } else {
+		$structures ++;
+		if ($scanning != 0) {
+		    &abort_scan ($filename)
+		} else {
+		    if ($opt_v) {
+			print "FUNCTION Start found at line $line.\n";
+		    }
+
+		    $scanning ++;
+		    $start_line = $line;
+		    $dat = $';
+		}
+	    }
+	    next;
+	}
+
+	if (/(SIDOC|DANG)_END_STRUCT/) {
+	    # Function End Found.
+	    if ($structures != 1) {
+		print "No Function Start found (END at line $line). Ignoring.\n";
+	    } else {
+		if ($opt_v) {
+		    print "FUNCTION end found at line $line.\n";
+		}
+
+		$structures --;
+#this also prefixes '/*' if present :-(		$dat .= $`;
+		push (@structures, &strip_comments ($dat));
+		undef $dat;
+		$scanning --;
+	    }
+	    next;
+	}
+
+	if (/(SIDOC|DANG)_BEGIN_REMARK/) {
 	    # Remark Start Found ...
 	    if ($remarks != 0) {
 		print "Already Found a Remark start at line $start_line. Ignoring.\n";
@@ -518,7 +653,7 @@ sub scan_file  {
 	    next;
 	}
 
-	if (/DANG_END_REMARK/) {
+	if (/(SIDOC|DANG)_END_REMARK/) {
 	    # Remark End Found.
 	    if ($remarks != 1) {
 		print "No Remark Start found (END at line $line). Ignoring.\n";
@@ -536,7 +671,7 @@ sub scan_file  {
 	    next;
 	}
 
-	if (/DANG_BEGIN_NEWIDEA/) {
+	if (/(SIDOC|DANG)_BEGIN_NEWIDEA/) {
 	    # NEWIDEA Start Found ...
 	    if ($newideas != 0) {
 		print "Already Found a NewIdea start at line $start_line. Ignoring.\n";
@@ -557,7 +692,7 @@ sub scan_file  {
 	    next;
 	}
 
-	if (/DANG_END_NEWIDEA/) {
+	if (/(SIDOC|DANG)_END_NEWIDEA/) {
 	    # NEWIDEA End Found.
 	    if ($newideas != 1) {
 		print "No NewIdeas Start found (END at line $line). Ignoring.\n";
@@ -575,7 +710,7 @@ sub scan_file  {
 	    next;
 	}
 
-	if (/DANG_FIXTHIS/) {
+	if (/(SIDOC|DANG)_FIXTHIS/) {
 	    # Fixthis Found ...
 	    if ($scanning != 0) {
 		die "Alreading Scanning an Item - Missing end (started at line $start_line.) Aborting\n";
@@ -592,7 +727,6 @@ sub scan_file  {
 	}
 
 	$dat .= $_;
-
     }
     close INPUT;
 
@@ -604,6 +738,7 @@ sub scan_file  {
     &handle_modules ($filename);
     &handle_changes ($filename);
     &handle_functions ($filename);
+    &handle_structures ($filename);
     &handle_remarks ($filename);
     &handle_fixthis ($filename);
     &handle_newideas ($filename);
@@ -623,6 +758,9 @@ sub abort_scan {
     if ($functions == 1) {
 	die ("At line $line in $filename I found a new marker.\nI was already scanning a FUNCTION section which started at line $start_line.\n*** ABORTING ***\n");
     }
+    if ($structures == 1) {
+	die ("At line $line in $filename I found a new marker.\nI was already scanning a FUNCTION section which started at line $start_line.\n*** ABORTING ***\n");
+    }
     if ($remarks == 1) {
 	die ("At line $line in $filename I found a new marker.\nI was already scanning a REMARKS section which started at line $start_line.\n*** ABORTING ***\n");
     }
@@ -644,19 +782,70 @@ sub strip_comments {
     $save = $*;
     $* = 1;
 
+#             / */ \s* \n \s* /*              / \n  /g
     $text =~ s#\*/\s*\n\s*/\*#\n#g;
     $text =~ s#^\s*//##g;
+
+    $text =~ s#^\s*\*/\s*\n##g;
+    $text =~ s#^\s*/\*\s*\n##g;
+    $text =~ s#^\s*/\*\s*##g;
+#print "AAAAAAA $text BBBBBBB\n";
+
     $text =~ s#^\s*\*##g;
     
     $text =~ s#^\s*# #;
     $text =~ s#^\n# \n#;
-    $text =~ s#\s*\*//?$# #;
+#    $text =~ s#\s*\*//?$# #;
 
     $* = $save;
 
     return &protect_specials ($text);
 }
 
+sub handle_subremarks {
+    local ($dat,$all) = @_;
+    local (@data);
+    local ($inverb, $inremark);
+
+    @data = split (/\n/, $dat);
+    $inverb = 0;
+    $inremark = 0;
+    foreach $data (@data) {
+	if ($data =~ /^\s*VERB\s*/) {
+	    print OUTPUT "<p><verb>\n";
+	    $inverb =1;
+	    next;
+	}
+	if ($inverb) {
+	    if ($data =~ /^\s*\/VERB\s*/) {
+		print OUTPUT "</verb>\n";
+		$inverb =0;
+	    }
+	    else {
+		$data = un_protect_specials($data);
+		print OUTPUT "$data\n";
+	    }
+	    next;
+	}
+	if ($all) {
+	    print OUTPUT "$data\n";
+	    next;
+	}
+	if ($data =~ /^\s*REMARK\s*/) {
+	    $inremark =1;
+	    next;
+	}
+	if ($inremark) {
+	    if ($data =~ /^\s*\/REMARK\s*/) {
+		$inremark =0;
+	    }
+	    else {
+		print OUTPUT "$data\n";
+	    }
+	    next;
+	}
+    }    
+}
 
 sub handle_modules {
     local ($filename) = @_;
@@ -668,6 +857,7 @@ sub handle_modules {
 
     if (defined @modules) {
 	foreach $mod (@modules) {
+	    &handle_subremarks($mod,0);
 	    if ($mod =~ /maintainer:/io) {
 		if ($opt_v) {
 		    print "Found maintainer details ...\n";
@@ -675,7 +865,7 @@ sub handle_modules {
 		if ($count > 0) {
 		    print OUTPUT "-----\n\n", $`, "\n";
 		}
-		print OUTPUT "<sect2>Maintainers\n<p>\n";
+		print OUTPUT "<P>Maintainers:\n<p>\n";
 		$maints = $';
 		while ($maints =~ /\s*(.*)\s*&lt;(.*)&gt;/g) {
 		    $addr = $2;
@@ -715,12 +905,14 @@ sub handle_functions {
     local (@nodes);
     local ($this, $count, @data, $data);
     local ($inargslist, @args);
+    local ($inverb,$inproto,$skip);
 
     if (defined @functions) {
-	print OUTPUT "<sect2>Functions in $filename\n\n<p>\n";
+	print OUTPUT "<sect1>Functions in $filename\n\n<p>\n";
 
   	foreach (@functions) {
-	    /\s*(\w*)/;
+#	    /\s*(\w*)/;
+	    /\s*(.*)\n/;
 	    push (nodes, $1);
 	}
 
@@ -730,18 +922,69 @@ sub handle_functions {
 	$* = 1;
 	
 	while ($this = shift @nodes ) {
-	    print OUTPUT "<sect3>$this\n\n\n<p>\n";
+	    print OUTPUT "<sect2>$this\n\n\n<p>\n";
 	    @data = split (/\n/, shift (@functions));
 	    shift @data;	# Ignore the first line - the function name
 	    $inargslist = 0;
 	    $inretlist = 0;
 	    undef @args;
 
+	    $inverb = 0;
+	    $inproto = 0;
+	    $skip = 0;
 	    foreach $data (@data) {
+		if ($data =~ /^\s*(\/*)SKIP\s*/) {
+		    if ($1 eq "/") {$skip = 0;}
+		    else {$skip = 1;}
+		    next;
+		}
+		if ($skip) {
+		    if ($data =~ /^\s*\/*VERB\s*|^\s*\/*PROTO\s*/) {
+			$skip = 0;
+		    }
+		    else {next;}
+		}
+		if ($data =~ /^\s*VERB\s*/) {
+		    print OUTPUT "<p><verb>\n";
+		    $inverb =1;
+		    next;
+		}
+		if ($inverb) {
+		    if ($data =~ /^\s*\/VERB\s*/) {
+			print OUTPUT "</verb>\n";
+			$inverb =0;
+		    }
+		    else {
+			$data = un_protect_specials($data);
+			print OUTPUT "$data\n";
+		    }
+		    next;
+		}
+		if ($data =~ /^\s*PROTO\s*/) {
+		    if ($inproto) {print OUTPUT "</verb>\n";}
+		    print OUTPUT "<p><verb>\n";
+		    $inproto =1;
+		    next;
+		}
+		if ($inproto) {
+		    if ($data =~ /^\s*\{|^\s*\/PROTO\s*/) {
+			print OUTPUT "</verb>\n";
+			$inproto =0;
+			$skip = 1;
+		    }
+		    else {
+			$data = un_protect_specials($data);
+			print OUTPUT "$data\n";
+		    }
+		    next;
+		}
+		if ($data =~ /^\s*\/PROTO\s*/) {
+			$inproto =0;
+			next;
+		}
  		if ($data =~ /^\s*$/) {	# Blank line ....
-		    next;	        # .... skip
+		    if ($inargslist || $inretlist) {next;} # .... skip
 		}
-
 		if ($data =~ /return:/io) { # Want to treat data as return
 		    print OUTPUT "<p>Return values mean:&nl;\n";
 		    $inretlist = 1;
@@ -797,11 +1040,143 @@ sub handle_functions {
 	
 }
 
+sub handle_structures {
+    local ($filename) = @_;
+    local (@nodes);
+    local ($this, $count, @data, $data);
+    local ($inargslist, @args, $arg, $insubarg);
+    local ($inverb) = 0;
+
+    if (defined @structures) {
+	print OUTPUT "<sect1>Data Definitions in $filename\n\n<p>\n";
+
+  	foreach (@structures) {
+	    /\s*(.*)\n/;
+	    push (nodes, $1);
+	}
+
+	print OUTPUT "These are the structures and/or data defined in $filename.\n\n";
+    
+	$save = $*;
+	$* = 1;
+	
+	while ($this = shift @nodes ) {
+	    print OUTPUT "<sect2>$this\n\n\n<p>\n";
+	    @data = split (/\n/, shift (@structures));
+	    shift @data;	# Ignore the first line - the header line
+	    $inargslist = 0;
+	    $inretlist = 0;
+	    undef @args;
+
+	    $inverb = 0;
+	    foreach $data (@data) {
+		if ($data =~ /^\s*VERB\s*/) {
+		    print OUTPUT "<p><verb>\n";
+		    $inverb =1;
+		    next;
+		}
+		if ($inverb) {
+		    if ($data =~ /^\s*\/VERB\s*/) {
+			print OUTPUT "</verb>\n";
+			$inverb =0;
+		    }
+		    else {
+			$data = un_protect_specials($data);
+			print OUTPUT "$data\n";
+		    }
+		    next;
+		}
+ 		if ($data =~ /^\s*$/) {	# Blank line ....
+		    if ($inargslist) {next;}	        # .... skip
+		}
+
+		if ($data =~ /elements:/io) { # Want to treat data as args
+		    print OUTPUT "<p>Elements are:&nl;\n";
+		    $inargslist = 1;
+		    next;
+		}
+
+		if ($data =~ /description:/io) { # Descriptive (default)
+		    $inargslist = 0;
+		    next;
+ 		}
+
+		if ($inargslist) { # storing list items
+		    push (args, $data);
+		    next;
+		} 
+
+		if (defined @args) { # no longer storing - time to output
+		    print OUTPUT "<itemize>\n";
+		    $insubarg =0;
+		    foreach $arg (@args) {
+			if ($insubarg) {
+			    if ($arg =~ /\*\//) {
+				print OUTPUT "$`\n";
+				$insubarg = 0;
+				next;
+			    }
+			    print OUTPUT $arg, "\n";
+			    next;
+			}
+			if ($arg =~ /\/\*/) {
+			    print OUTPUT "<ITEM> ", $`, "\n\n";
+			    $arg = $';
+			    if ($arg =~ /\*\//) {$arg = $`;}
+			    else {$insubarg = 1;}
+			    print OUTPUT  $arg, "\n";
+			    next;
+			}
+			print OUTPUT "<ITEM> ", $arg, "\n";
+		    }
+		    print OUTPUT "</itemize>\n";
+		    undef @args;
+		}
+		print OUTPUT $data, "\n";
+		
+	    }
+
+	    if (defined @args) { # no longer storing - time to output
+		print OUTPUT "<itemize>\n";
+		$insubarg = 0;
+		foreach $arg (@args) {
+			if ($insubarg) {
+			    if ($arg =~ /\*\//) {
+				print OUTPUT "$`\n";
+				$insubarg = 0;
+				next;
+			    }
+			    print OUTPUT $arg, "\n";
+			    next;
+			}
+			if ($arg =~ /\/\*/) {
+			    print OUTPUT "<ITEM> ", $`, "<p>\n";
+			    $arg = $';
+			    if ($arg =~ /\*\//) {$arg = $`;}
+			    else {$insubarg = 1;}
+			    print OUTPUT  $arg, "\n";
+			    next;
+			}
+			print OUTPUT "<ITEM> ", $arg, "\n";
+		}
+		print OUTPUT "</itemize>\n";
+	    }
+
+	    print OUTPUT "\n\n";
+	} 
+
+	undef @structures;
+    } elsif ($opt_i)  {
+	print OUTPUT "We appear to have no information on the structures in $filename.\n\n";
+    }
+	
+}
+
 sub handle_newideas {
     local ($filename) = @_;
 
     if (defined @newideas) {
-	print OUTPUT "<sect2>New Ideas for $filename\n\n<p>\n";
+	print OUTPUT "<sect1>New Ideas for $filename\n\n<p>\n";
     
 	print OUTPUT (shift @newideas), "\n";
 
@@ -820,12 +1195,15 @@ sub handle_remarks {
     local ($filename) = @_;
 
     if (defined @remarks) {
-	print OUTPUT "<sect2>Remarks in $filename\n\n<p>\n";
+	print OUTPUT "<sect1>Remarks in $filename\n\n<p>\n";
     
-	print OUTPUT (shift @remarks), "\n";
+	&handle_subremarks(shift @remarks,1);
+#	print OUTPUT (shift @remarks), "\n";
 
 	foreach (@remarks) {
-	    print OUTPUT "-----\n\n", $_, "\n";
+#	    print OUTPUT "-----\n\n", $_, "\n";
+	    print OUTPUT "-----\n\n";
+	    &handle_subremarks($_,1);
 	}
 
 	undef @remarks;
@@ -839,7 +1217,7 @@ sub handle_fixthis {
     local ($filename) = @_;
 
     if (defined @fixthis) {
-	print OUTPUT "<sect2>Items for Fixing in $filename\n\n<p>\n";
+	print OUTPUT "<sect1>Items for Fixing in $filename\n\n<p>\n";
 
 	print OUTPUT (shift @fixthis), "\n";
 
