@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -26,14 +26,12 @@ enum { ES_INDEX = 0, CS_INDEX = 1, SS_INDEX = 2,  DS_INDEX = 3,
 static struct vm86_regs SAVED_REGS;
 static struct vm86_regs MOUSE_SAVED_REGS;
 static struct vm86_regs VIDEO_SAVED_REGS;
-static struct vm86_regs INT15_SAVED_REGS;
 #define S_REG(reg) (SAVED_REGS.##reg)
 #endif
 #ifdef __NetBSD__
 static struct sigcontext SAVED_REGS;
 static struct sigcontext MOUSE_SAVED_REGS;
 static struct sigcontext VIDEO_SAVED_REGS;
-static struct sigcontext INT15_SAVED_REGS;
 #define S_REG(reg) (SAVED_REGS.##reg)
 #endif
 
@@ -145,12 +143,8 @@ static int inline is_dos_selector(unsigned short sel)
 {
     unsigned long base = Segments[sel >> 3].base_addr;
 
-    if (base > 0x10ffef) {	/* ffff:ffff for DOS high */
-      D_printf("DPMI: base address %#lx of sel %#x > DOS limit\n", base, sel);
-      return 0;
-    } else
-    if (base & 0xf) {
-      D_printf("DPMI: base address %#lx of sel %#x not para. aligned.\n", base, sel);
+    if ( (base > 0xffff0) || (base & 0xf)) {
+      D_printf("DPMI: base adddress > 1MB or not para. aligned.\n");
       return 0;
     } else
       return 1;
@@ -173,7 +167,7 @@ msdos_pre_extender(struct sigcontext_struct *scp, int intr)
 {
     /* only consider DOS and some BIOS services */
     switch (intr) {
-    case 0x10: case 0x15:
+    case 0x10:
     case 0x20: case 0x21:
     case 0x25: case 0x26:
     case 0x33:			/* mouse function */
@@ -242,25 +236,10 @@ msdos_pre_extender(struct sigcontext_struct *scp, int intr)
 	    else
 		return 0;
 	    break;
-	case 0x4f:		/* vesa functions */
-	    switch (_LO(ax)) {
-		case 0x00:	/* return VBE controller info */
-		case 0x01:	/* return VBE mode info */
-		case 0x04:	/* save/restore state */
-		case 0x09:	/* set/get palette data */
-		    ES_MAPPED = 1;
-		    break;
-		default:
-		    return 0;
-	    }
-	    break;
 	default:
 	    return 0;
 	}
 	break;
-    case 0x15:			/* misc */
-	INT15_SAVED_REGS = REGS;
-	return 0;
     case 0x20:			/* DOS terminate */
 	return old_dos_terminate(scp);
     case 0x21:
@@ -582,13 +561,11 @@ msdos_pre_extender(struct sigcontext_struct *scp, int intr)
 	    if ((_HI(ax) == 0x4e) && (_ecx & 0x8))
 		D_printf("DPMI: MS-DOS try to find volume label\n");
 	    if ( !is_dos_selector(_ds)) {
-		char *src, *dst;
+		D_printf("DPMI: passing ASCIIZ > 1MB to dos\n"); 
 		REG(ds) = DPMI_private_data_segment+DPMI_private_paragraphs;
-		src = (char *)(GetSegmentBaseAddress(_ds)+(DPMIclient_is_32 ? _edx : (_LWORD(edx))));
-		dst = (char *)((REG(ds) << 4)+LWORD(edx));
-		D_printf("DPMI: passing ASCIIZ > 1MB to dos %#x\n", (int)dst); 
-		D_printf("%#x: '%s'\n", (int)src, src);
-		strcpy(dst, src);
+		strcpy((void *)((REG(ds) << 4)+LWORD(edx)),
+		       (void *)GetSegmentBaseAddress(_ds)+
+		       (DPMIclient_is_32 ? _edx : (_LWORD(edx))));
 	    }
 	    in_dos_21++;
 	    return 0;
@@ -719,13 +696,11 @@ msdos_pre_extender(struct sigcontext_struct *scp, int intr)
 	    return 0;
 	case 0x6c:		/*  Extended Open/Create */
 	    if ( !is_dos_selector(_ds)) {
-		char *src, *dst;
+		D_printf("DPMI: passing ASCIIZ > 1MB to dos\n"); 
 		REG(ds) = DPMI_private_data_segment+DPMI_private_paragraphs;
-		src = (char *)(GetSegmentBaseAddress(_ds)+(DPMIclient_is_32 ? _esi : (_LWORD(esi))));
-		dst = (char *)((REG(ds) << 4)+LWORD(esi));
-		D_printf("DPMI: passing ASCIIZ > 1MB to dos %#x\n", (int)dst); 
-		D_printf("%#x: '%s'\n", (int)src, src);
-		strcpy(dst, src);
+		strcpy((void *)((REG(ds) << 4)+LWORD(esi)),
+		       (void *)GetSegmentBaseAddress(_ds)+
+		       (DPMIclient_is_32 ? _esi : (_LWORD(esi))));
 	    }
 	    in_dos_21++;
 	    return 0;
@@ -778,36 +753,26 @@ msdos_pre_extender(struct sigcontext_struct *scp, int intr)
 
     if (DS_MAPPED) {
 	if ( !is_dos_selector(_ds)) {
-	    char *src, *dst;
-	    int len;
+	    D_printf("DPMI: whole segment of DS copy to DOS.\n");
 	    DS_MAPPED = _ds;
 	    REG(ds) =   DPMI_private_data_segment+DPMI_private_paragraphs;
-	    src = (char *)GetSegmentBaseAddress(_ds);
-	    dst = (char *)(REG(ds)<<4);
-	    len = ((Segments[_ds >> 3].limit > 0xffff) ||
-	    	Segments[_ds >> 3].is_big) ?
-		0xffff : Segments[_ds >> 3].limit;
-	    D_printf("DPMI: whole segment of DS at %#x copy to DOS at %#x for %#x\n",
-		(int)src, (int)dst, len);
-	    memmove (dst, src, len);
+	    memmove ((void *)(REG(ds)<<4), (void *)GetSegmentBaseAddress(_ds),
+		    ((Segments[_ds >> 3].limit > 0xffff) ||
+		     Segments[_ds >> 3].is_big) ?
+		    0xffff : Segments[_ds >> 3].limit);
 	} else
 	    DS_MAPPED = 0;
     }
 
     if (ES_MAPPED) {
 	if ( !is_dos_selector(_es)) {
-	    char *src, *dst;
-	    int len;
+	    D_printf("DPMI: whole segment of ES copy to DOS.\n");
 	    ES_MAPPED = _es;
 	    REG(es) =   DPMI_private_data_segment+DPMI_private_paragraphs;
-	    src = (char *)GetSegmentBaseAddress(_es);
-	    dst = (char *)(REG(es)<<4);
-	    len = ((Segments[_es >> 3].limit > 0xffff) ||
-	    	Segments[_es >> 3].is_big) ?
-		0xffff : Segments[_es >> 3].limit;
-	    D_printf("DPMI: whole segment of ES at %#x copy to DOS at %#x for %#x\n",
-		(int)src, (int)dst, len);
-	    memmove (dst, src, len);
+	    memmove ((void *)(REG(es)<<4), (void *)GetSegmentBaseAddress(_es),
+		    ((Segments[_es >> 3].limit > 0xffff)||
+		     Segments[_ds >> 3].is_big) ?
+		    0xffff : Segments[_es >> 3].limit);
 	} else
 	    ES_MAPPED = 0;
     }
@@ -863,16 +828,16 @@ msdos_post_extender(int intr)
 	} else
 	    break;
     case 0x15:
-	/* we need to save regs at int15 because AH has the return value */
-	if ((INT15_SAVED_REGS.eax & 0xff00) == 0xc000) { /* Get Configuration */
+      switch(HI(ax)) {
+        case 0xc0:      /* Get Configuartion */
                 if (REG(eflags)&CF)
                         return;
                 if (!(dpmi_stack_frame[current_client].es =
-                         ConvertSegmentToDescriptor(REG(es)))) return;
-                break;
-      }
-        else
+                         ConvertSegmentToDescriptor(REG(es))))
                 return;
+        default:
+                return;
+      }
     /* only copy buffer for dos services */
     case 0x21:
     case 0x25: case 0x26:
@@ -893,33 +858,23 @@ msdos_post_extender(int intr)
 
     if (DS_MAPPED) {
 	unsigned short my_ds;
-	char *src, *dst;
-	int len;
 	my_ds =   DPMI_private_data_segment+DPMI_private_paragraphs;
-	src = (char *)(my_ds<<4);
-	dst = (char *)GetSegmentBaseAddress(DS_MAPPED);
-	len = ((Segments[DS_MAPPED >> 3].limit > 0xffff) ||
-	    	Segments[DS_MAPPED >> 3].is_big) ?
-		0xffff : Segments[DS_MAPPED >> 3].limit;
-	D_printf("DPMI: DS_MAPPED seg at %#x copy back at %#x for %#x\n",
-		(int)src, (int)dst, len);
-	memmove (dst, src, len);
+	memmove ((void *)GetSegmentBaseAddress(DS_MAPPED), (void *)(my_ds<<4), 
+		((Segments[DS_MAPPED >> 3].limit > 0xffff)||
+		 Segments[DS_MAPPED >> 3].is_big) ?
+		0xffff : Segments[DS_MAPPED >> 3].limit);
     } 
+    DS_MAPPED = 0;
 
     if (ES_MAPPED) {
 	unsigned short my_es;
-	char *src, *dst;
-	int len;
 	my_es =   DPMI_private_data_segment + DPMI_private_paragraphs;
-	src = (char *)(my_es<<4);
-	dst = (char *)GetSegmentBaseAddress(ES_MAPPED);
-	len = ((Segments[ES_MAPPED >> 3].limit > 0xffff) ||
-	    	Segments[ES_MAPPED >> 3].is_big) ?
-		0xffff : Segments[ES_MAPPED >> 3].limit;
-	D_printf("DPMI: ES_MAPPED seg at %#x copy back at %#x for %#x\n",
-		(int)src, (int)dst, len);
-	memmove (dst, src, len);
+	memmove ((void *)GetSegmentBaseAddress(ES_MAPPED), (void *)(my_es<<4), 
+		((Segments[ES_MAPPED >> 3].limit > 0xffff) ||
+		Segments[ES_MAPPED >> 3].is_big) ?
+		 0xffff : Segments[ES_MAPPED >> 3].limit);
     } 
+    ES_MAPPED = 0;
 	       
     switch (intr) {
     case 0x21:
@@ -934,14 +889,14 @@ msdos_post_extender(int intr)
 		 GetSegmentBaseAddress(dpmi_stack_frame[current_client].cs)>>4;
 	    /* restore parent PSP */
 	    *(unsigned short *)((char *)(CURRENT_PSP<<4) + 0x16) = PARENT_PSP;
-	    break;
+	    return;
 	case 0x09:		/* print String */
 	case 0x1a:		/* set DTA */
 	    dpmi_stack_frame[current_client].edx = S_REG(edx);
-	    break;
+	    return;
 	case 0x11: case 0x12:	/* findfirst/next using FCB */
 	    memmove(user_FCB, SEG_ADR((void *), ds, dx), 0x50);
-	    break ;
+	    return ;
 	case 0x29:		/* Parse a file name for FCB */
 	    {
 		unsigned short seg =
@@ -962,24 +917,24 @@ msdos_post_extender(int intr)
 			    (void *)((REG(es)<<4) + LWORD(edi)),  0x50);
 		}
 	    }
-	    break;
+	    return;
 	case 0x2f:		/* GET DTA */
 	case 0x34:		/* Get Address of InDOS Flag */
 	case 0x35:		/* GET Vector */
 	case 0x52:		/* Get List of List */
-	    if (!ES_MAPPED)
-	      dpmi_stack_frame[current_client].es =
+	    dpmi_stack_frame[current_client].es =
 	               ConvertSegmentToDescriptor(REG(es));
 	    break;
 
         case 0x59:		/* Get EXTENDED ERROR INFORMATION */
-	    if(LWORD(eax) == 0x22 && !ES_MAPPED) { /* only this code has a pointer */
+	    if(LWORD(eax) == 0x22) { /* only this code has a pointer */
 		dpmi_stack_frame[current_client].es =
 			ConvertSegmentToDescriptor(REG(es));
 	    }
 	    break;
+
 	case 0x38:
-	    if (S_LO(ax) == 0x00 && !DS_MAPPED) { /* get country info */
+	    if (S_LO(ax) == 0x00) { /* get contry info */
 		dpmi_stack_frame[current_client].ds =
 	               ConvertSegmentToDescriptor(REG(ds));
 	    }
@@ -1027,7 +982,7 @@ msdos_post_extender(int intr)
 #endif 		
 		dpmi_stack_frame[current_client].ebx = ConvertSegmentToDescriptor(psp);
 	    }
-	    break;
+	    return;
 	case 0x53:		/* Generate Drive Parameter Table  */
 	    {
 		unsigned short seg =
@@ -1043,21 +998,21 @@ msdos_post_extender(int intr)
 			    0x60);
 		}
 	    }
-	    break ;
+	    return ;
 	case 0x56:		/* rename */
 	    dpmi_stack_frame[current_client].edx = S_REG(edx);
 	    dpmi_stack_frame[current_client].edi = S_REG(edi);
-	    break ;
+	    return ;
 	case 0x5d:
 	    if (S_LO(ax) == 0x06) /* get address of DOS swappable area */
 				/*        -> DS:SI                     */
 		dpmi_stack_frame[current_client].ds = ConvertSegmentToDescriptor(REG(ds));
-	    break;
+	    return;
 	case 0x3f:
 	    if (READ_DS_COPIED) {
 		dpmi_stack_frame[current_client].edx = S_REG(edx);
 		if (LWORD(eflags) & CF)
-		    break;
+		    return;
 		memmove((void *)(GetSegmentBaseAddress
 				(dpmi_stack_frame[current_client].ds)
 			 + (DPMIclient_is_32 ? S_REG(edx) : (S_LWORD(edx)))),
@@ -1073,7 +1028,7 @@ msdos_post_extender(int intr)
 	case 0x5f:		/* redirection */
 	    switch (S_LO(ax)) {
 	    case 0: case 1:
-		break ;
+		return ;
 	    case 2 ... 6:
 		dpmi_stack_frame[current_client].esi = S_REG(esi);
 		memmove ((void *)GetSegmentBaseAddress
@@ -1090,7 +1045,7 @@ msdos_post_extender(int intr)
 	    }
 	    break;
 	default:
-	    break;
+	    return;
 	}
 	break;
     case 0x25:			/* Absolute Disk Read */
@@ -1108,25 +1063,23 @@ msdos_post_extender(int intr)
 	      (dpmi_stack_frame[current_client].esp&0xffff)) =
 		 LWORD(eflags);
 	}
-	break;
+	return;
     case 0x33:			/* mouse */
 	switch (MOUSE_SAVED_REGS.eax & 0xffff) {
 	case 0x14:		/* swap call back */
 	    dpmi_stack_frame[current_client].es =
                   	    ConvertSegmentToDescriptor(REG(es)); 
-	    break;
+	    return;
 	case 0x19:		/* Get User Alternate Interrupt Address */
 	    dpmi_stack_frame[current_client].ebx =
                   	    ConvertSegmentToDescriptor(LWORD(ebx)); 
-	    break;
+	    return;
 	default:
-	    break;
+	    return;
 	}
     default:
-	break;
+	return;
     }
-    DS_MAPPED = 0;
-    ES_MAPPED = 0;
 }
 
 static char decode_use_16bit;
