@@ -212,6 +212,11 @@ void cpu_setup(void)
   REG(eflags) |= (VIF | VIP);
 #endif
 
+#ifdef X86_EMULATOR
+  if (config.cpuemu) {
+    init_cpu();
+  }
+#endif
 }
 
 
