@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -46,9 +46,6 @@
 #endif
 #include <sys/stat.h>
 #include <sys/ioctl.h>
-#if GLIBC_VERSION_CODE >= 2000
-  #include <sys/io.h>
-#endif
 
 #include "emu.h"
 #include "port.h"
@@ -642,10 +639,12 @@ static Bit8u special_port_inb(ioport_t port)
 		return ((res & 0xfd) | (hgc_Konv & 0x02));
 	    }
 	}
+#if X_GRAPHICS
 	if ((port==0x3ba)||(port==0x3da)) {
 		res = Attr_get_input_status_1();
 	}
 	else
+#endif
 	if (port==0x3db)	/* light pen strobe reset */
 		res = 0;
 	return res;
@@ -901,10 +900,8 @@ int extra_port_init(void)
 	SET_HANDLE_COND(0x3db,HANDLE_SPECIAL);		/* R */
 
 	i = READ_WORD(BIOS_VIDEO_PORT);
-	if (i) {	/* !config.vga */
-	    SET_HANDLE_COND(i,HANDLE_SPECIAL);		/* W */
-	    SET_HANDLE_COND(i+1,HANDLE_SPECIAL);	/* W */
-	}
+	SET_HANDLE_COND(i,HANDLE_SPECIAL);		/* W */
+	SET_HANDLE_COND(i+1,HANDLE_SPECIAL);		/* W */
 
  	return 0;
 }
