@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -89,7 +89,7 @@ __asm__("___START___: jmp _emulate\n");
 #include "netbsd_vm86.h"
 #endif
 #ifdef __linux__
-#if GLIBC_VERSION_CODE >= 2000
+#if __GLIBC__ > 1
 #include <sys/vt.h>
 #include <sys/kd.h>
 #else
@@ -159,6 +159,11 @@ extern void     print_version(void);
 
 extern void io_select_init(void);
 
+#ifdef USE_THREADS
+extern void treads_init(void);
+#endif
+
+
 
 void 
 boot(void)
@@ -377,10 +382,10 @@ changesegs()
  * 
  */
 #ifdef __ELF__
-int
+void 
 main(int argc, char **argv)
 #else
-int
+void 
 emulate(int argc, char **argv)
 #endif
 {
@@ -391,6 +396,14 @@ emulate(int argc, char **argv)
 
     srand(time(NULL));
 
+#ifdef USE_THREADS
+    treads_init();		/* init the threads system,
+				 * after this we will be thread0.
+				 * Because treads_init() captures SIGTERM
+				 * and signal_init() also does it,
+				 * we capture it via 'exit'.
+    				 */
+#endif
     /* NOW! it is safe to touch the priv code.  */
     priv_init();  /* This must come first! */
 
@@ -468,7 +481,6 @@ emulate(int argc, char **argv)
     sync();
     fprintf(stderr, "Not a good day to die!!!!!\n");
     leavedos(99);
-    return 0;  /* just to make gcc happy */
 }
 
 #if 0    /* disable C-A-D until someone will fix it (if really needed) */
@@ -639,7 +651,14 @@ leavedos(int sig)
        if (command) run_unix_command(command); /* beware of system() !! */
     }
 
+#ifdef USE_THREADS
+    {
+	extern void Exit(int status);
+	Exit(sig);
+    }
+#else
     _exit(sig);
+#endif
 }
 
 #if 0
