@@ -54,6 +54,9 @@ struct debug_flags {
    pd,			/* Packet driver     "P" */
    request,		/* PIC               "r" */
    sound 		/* SOUND	     "S" */
+#ifdef X86_EMULATOR
+   ,emu			/* CPU emulation     "e" */
+#endif
    ;
 };
 
@@ -116,6 +119,9 @@ void verror(const char *fmt, va_list args);
 #define pd_printf(f,a...)       ifprintf(d.pd,f,##a)	   /* pktdrvr */
 #define r_printf(f,a...)        ifprintf(d.request,f,##a)
 #define S_printf(f,a...)     	ifprintf(d.sound,f,##a)
+#ifdef X86_EMULATOR
+#define e_printf(f,a...)     	ifprintf(d.emu,f,##a)
+#endif
 
 #define ALL_DEBUG_ON	memset(&d,9,sizeof(d))
 #define ALL_DEBUG_OFF	memset(&d,0,sizeof(d))
