@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -14,10 +14,10 @@
 
 #include <features.h>
 #include <sys/types.h>
-#if GLIBC_VERSION_CODE == 2000
+#include <signal.h> 
+#if __GLIBC__ == 2 && __GLIBC_MINOR__ == 0
 #include <sigcontext.h>
 #endif
-#include <signal.h> 
 #include "config.h"
 #include "types.h"
 #include "extern.h"
@@ -135,7 +135,11 @@ void getKeys(void);
 
      int set_ioperm(int, int, int);
 
+#ifdef X86_EMULATOR
+EXTERN struct debug_flags d INIT({0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0});
+#else
 EXTERN struct debug_flags d INIT({0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0});
+#endif
  
 EXTERN u_char in_sighandler, in_ioctl;
 /* one-entry queue ;-( for ioctl's */
@@ -219,6 +223,10 @@ typedef struct vesamode_type_struct {
      typedef struct config_info {
        int hdiskboot;
 
+#ifdef X86_EMULATOR
+       boolean cpuemu;
+       int emuspeed;
+#endif
        /* for video */
        boolean console;
        boolean console_video;
@@ -329,23 +337,6 @@ typedef struct vesamode_type_struct {
        char *tty_lockfile;	/* Lock file pretext ie LCK.. */
        boolean tty_lockbinary;	/* Binary lock files ? */
 
-       /* list of arbitrary features
-        * (at minimum 1, will be increased when needed)
-        * The purpose of these parameters is to switch between code
-        * or code pieces that in principal do the same, but for some
-        * unknown reasons behave different between machines.
-        * If a 'features' becomes obsolete (problem solved) it will
-        * remain dummy for a while before re-used.
-        *
-        * NOTE: 'features' are not subject to permanent documentation!
-        *       They should be considered 'temporary hacks'
-        *
-        * Currently assigned:
-        *
-        *   features[0]		use new(0) or old(1) Slangcode
-        */
-       int features[1];
-
        /* Sound emulation */
        __u16 sb_base;
        __u8 sb_dma;
@@ -495,7 +486,7 @@ do { \
 			sigaction(sig, &sa, NULL);
 #endif
 #ifdef __linux__
-#if GLIBC_VERSION_CODE
+#ifdef __GLIBC__
 #define SignalHandler __sighandler_t
 #endif
 #define NEWSETQSIG(sig, fun)	sa.sa_handler = (__sighandler_t)fun; \
