@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -8,7 +8,7 @@
 #define _EMU_IODEV_H
 
 #include <features.h>
-#if GLIBC_VERSION_CODE >= 2000
+#if __GLIBC__ > 1 
 #include <sys/time.h>
 #else
 #include <linux/time.h>
