@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -27,7 +27,7 @@ extern t_shiftstate shiftstate;
 #define PRESS 1
 #define RELEASE 0
 
-#define KEYB_QUEUE_LENGTH (255)
+#define KEYB_QUEUE_LENGTH 15
 
 /* public function definitions */
 
@@ -66,7 +66,6 @@ void keyb_server_close();
 #define SYSRQ_PRESSED  0x4000
 #define ALT            (L_ALT|R_ALT)
 #define SHIFT          (L_SHIFT|R_SHIFT)
-#undef	CTRL
 #define CTRL           (L_CTRL|R_CTRL)
 
 
