@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -86,6 +86,12 @@ struct brkentry {
    char is_dpmi;
 };
 
+struct cmd_db {
+   char cmdname[12];
+   void (*cmdproc)(int, char *[]);
+};
+
+
 struct segoff {
   unsigned short off,seg;
 };
